/*
Module : ShellCtrl2DataSource.cpp
Purpose: Implementation for the CShellCtrl2DataSource class
Created: PJN / 05-12-2024
History: PJN / 05-12-2024 1. Initial creation

Copyright (c) 2024 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "ShellCtrl2DropTarget.h"
#include "ShellTreeCtrl2.h"
#include "ShellListCtrl2.h"


//////////////////// Defines / Locals /////////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif //#ifdef _DEBUG


//////////////////// Implementation ///////////////////////////////////////////

CShellCtrl2DropTarget::CShellCtrl2DropTarget(_In_ CShellTreeCtrl2* pTreeCtrl) : m_pTreeCtrl{pTreeCtrl}
{
  m_bUseDropTargetHelper = SUCCEEDED(m_DropTargetHelper.CoCreateInstance(CLSID_DragDropHelper, nullptr, CLSCTX_ALL));
}

CShellCtrl2DropTarget::CShellCtrl2DropTarget(_In_ CShellListCtrl2* pListCtrl): m_pListCtrl{pListCtrl}
{
  m_bUseDropTargetHelper = SUCCEEDED(m_DropTargetHelper.CoCreateInstance(CLSID_DragDropHelper, nullptr, CLSCTX_ALL));
}

#pragma warning(suppress: 26429)
DROPEFFECT CShellCtrl2DropTarget::OnDragEnter(CWnd* pWnd, COleDataObject* pDataObject, DWORD dwKeyState, CPoint point)
{
  if (m_pTreeCtrl != nullptr)
  {
    const DWORD dwEffect{m_pTreeCtrl->OnDragEnter(pDataObject, dwKeyState, point)};
    if (m_bUseDropTargetHelper)
      m_DropTargetHelper->DragEnter(pWnd->GetSafeHwnd(), pDataObject->GetIDataObject(FALSE), &point, dwEffect);
    return dwEffect;
  }
  else if (m_pListCtrl != nullptr)
  {
    const DWORD dwEffect{m_pListCtrl->OnDragEnter(pDataObject, dwKeyState, point)};
    if (m_bUseDropTargetHelper)
      m_DropTargetHelper->DragEnter(pWnd->GetSafeHwnd(), pDataObject->GetIDataObject(FALSE), &point, dwEffect);
    return dwEffect;
  }
  else
  {
    ASSERT(FALSE);
    return DROPEFFECT_NONE;
  }
}

#pragma warning(suppress: 26429)
DROPEFFECT CShellCtrl2DropTarget::OnDragOver(CWnd* /*pWnd*/, COleDataObject* pDataObject, DWORD dwKeyState, CPoint point)
{
  if (m_pTreeCtrl != nullptr)
  {
    const DWORD dwEffect{m_pTreeCtrl->OnDragOver(pDataObject, dwKeyState, point)};
    if (m_bUseDropTargetHelper)
      m_DropTargetHelper->DragOver(&point, dwEffect);
    return dwEffect;
  }
  else if (m_pListCtrl != nullptr)
  {
    const DWORD dwEffect{m_pListCtrl->OnDragOver(pDataObject, dwKeyState, point)};
    if (m_bUseDropTargetHelper)
      m_DropTargetHelper->DragOver(&point, dwEffect);
    return dwEffect;
  }
  else
  {
    ASSERT(FALSE);
    return DROPEFFECT_NONE;
  }
}

#pragma warning(suppress: 26429)
BOOL CShellCtrl2DropTarget::OnDrop(CWnd* /*pWnd*/, COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point)
{
  if (m_pTreeCtrl != nullptr)
  {
    const BOOL bSuccess{m_pTreeCtrl->OnDrop(pDataObject, dropEffect, point)};
    if (m_bUseDropTargetHelper)
      m_DropTargetHelper->Drop(pDataObject->GetIDataObject(FALSE), &point, dropEffect);
    return bSuccess;
  }
  else if (m_pListCtrl != nullptr)
  {
    const BOOL bSuccess{m_pListCtrl->OnDrop(pDataObject, dropEffect, point)};
    if (m_bUseDropTargetHelper)
      m_DropTargetHelper->Drop(pDataObject->GetIDataObject(FALSE), &point, dropEffect);
    return bSuccess;
  }
  else
  {
    ASSERT(FALSE);
    return FALSE;
  }
}

#pragma warning(suppress: 26429)
DROPEFFECT CShellCtrl2DropTarget::OnDropEx(CWnd* /*pWnd*/, COleDataObject* pDataObject, DROPEFFECT dropDefault, DROPEFFECT dropList, CPoint point)
{
  if (m_pTreeCtrl != nullptr)
  {
    const DWORD dwEffect{m_pTreeCtrl->OnDropEx(pDataObject, dropDefault, dropList, point)};
    if (m_bUseDropTargetHelper)
      m_DropTargetHelper->Drop(pDataObject->GetIDataObject(FALSE), &point, dwEffect);
    return dwEffect;
  }
  else if (m_pListCtrl != nullptr)
  {
    const DWORD dwEffect{m_pListCtrl->OnDropEx(pDataObject, dropDefault, dropList, point)};
    if (m_bUseDropTargetHelper)
      m_DropTargetHelper->Drop(pDataObject->GetIDataObject(FALSE), &point, dwEffect);
    return dwEffect;
  }
  else
  {
    ASSERT(FALSE);
    return DROPEFFECT_NONE;
  }
}

#pragma warning(suppress: 26429)
void CShellCtrl2DropTarget::OnDragLeave(CWnd* /*pWnd*/)
{
  if (m_pTreeCtrl != nullptr)
  {
    m_pTreeCtrl->OnDragLeave();
    if (m_bUseDropTargetHelper)
      m_DropTargetHelper->DragLeave();
  }
  else if (m_pListCtrl != nullptr)
  {
    m_pListCtrl->OnDragLeave();
    if (m_bUseDropTargetHelper)
      m_DropTargetHelper->DragLeave();
  }
  else
  {
    ASSERT(FALSE);
  }
}

#pragma warning(suppress: 26429)
DROPEFFECT CShellCtrl2DropTarget::OnDragScroll(CWnd* /*pWnd*/, DWORD dwKeyState, CPoint point)
{
  if (m_pTreeCtrl != nullptr)
    return m_pTreeCtrl->OnDragScroll(dwKeyState, point);
  else if (m_pListCtrl != nullptr)
    return m_pListCtrl->OnDragScroll(dwKeyState, point);
  else
  {
    ASSERT(FALSE);
    return DROPEFFECT_NONE;
  }
}

#pragma warning(suppress: 26429)
_Success_(return != false) bool CShellCtrl2DropTarget::GetFirstPathFromDataObject(_In_ COleDataObject* pDataObject, _Inout_ CString& sFirstPath)
{
  //Validate our parameters
  ASSERT(pDataObject != nullptr);

  const IDataObject* piDataObj{pDataObject->GetIDataObject(FALSE)};
  if (piDataObj == nullptr)
    return false;
  HGLOBAL hGlobal{pDataObject->GetGlobalData(CF_HDROP)};
  if (hGlobal == nullptr)
    return false;
  HDROP hDrop{static_cast<HDROP>(GlobalLock(hGlobal))};
  if (hDrop == nullptr)
  {
    GlobalFree(hGlobal);
    return false;
  }
  const UINT nCount{DragQueryFile(hDrop, UINT_MAX, nullptr, 0)};
  if (nCount == 0)
  {
    GlobalUnlock(hGlobal);
    GlobalFree(hGlobal);
    return false;
  }
  for (UINT i{0}; i<nCount; i++)
  {
    const UINT nCharSize{DragQueryFile(hDrop, i, nullptr, 0)};
    if (nCharSize > 0)
    {
      CString sDroppedFileName;
      DragQueryFile(hDrop, i, sDroppedFileName.GetBuffer(nCharSize), nCharSize + 1);
      sDroppedFileName.ReleaseBuffer();
      WIN32_FIND_DATA fd{};
      HANDLE hFind{FindFirstFile(sDroppedFileName, &fd)};
      if ((hFind != INVALID_HANDLE_VALUE))
      {
        FindClose(hFind);
        sFirstPath = sDroppedFileName;
        break;
      }
    }
  }
  GlobalUnlock(hGlobal);
  GlobalFree(hGlobal);
  return true;
}

CString CShellCtrl2DropTarget::GetVolumePathName(_In_ const CString& sPath)
{
  CString sVolume;
  ::GetVolumePathName(sPath, sVolume.GetBuffer(_MAX_PATH), _MAX_PATH);
  sVolume.ReleaseBuffer();
  return sVolume;
}
