/*
Module : ShellCtrl2DropTarget.h
Purpose: Interface for the CShellCtrl2DropTarget class used by the CShellTreeCtrl2 and CShellListCtrl2 classes
Created: PJN / 05-12-2024
History: PJN / 05-12-2024 1. Initial creation

Copyright (c) 2024 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#pragma once

#ifndef __SHELLCTRL2DROPTARGET_H__
#define __SHELLCTRL2DROPTARGET_H__

#ifndef SHELLCTRL2DROPTARGET_EXT_CLASS
#define SHELLCTRL2DROPTARGET_EXT_CLASS
#endif //#ifndef SHELLCTRL2DROPTARGET_EXT_CLASS


//////////////////// Classes //////////////////////////////////////////////////

class CShellTreeCtrl2;
class CShellListCtrl2;

//Class which implements IDropTarget in the tree or list control
class SHELLCTRL2DROPTARGET_EXT_CLASS CShellCtrl2DropTarget : public COleDropTarget
{
public:
//Constructors / Destructors
  CShellCtrl2DropTarget() = delete;
  CShellCtrl2DropTarget(_In_ CShellTreeCtrl2* pTreeCtrl);
  CShellCtrl2DropTarget(_In_ CShellListCtrl2* pListCtrl);
  CShellCtrl2DropTarget(const CShellCtrl2DropTarget&) = delete;
  CShellCtrl2DropTarget(CShellCtrl2DropTarget&&) = delete;
  ~CShellCtrl2DropTarget() = default; //NOLINT(modernize-use-override)

//Methods
  CShellCtrl2DropTarget& operator=(const CShellCtrl2DropTarget&) = delete;
  CShellCtrl2DropTarget& operator=(CShellCtrl2DropTarget&&) = delete;
  DROPEFFECT OnDragEnter(CWnd* pWnd, COleDataObject* pDataObject, DWORD dwKeyState, CPoint point) override;
  DROPEFFECT OnDragOver(CWnd* pWnd, COleDataObject* pDataObject, DWORD dwKeyState, CPoint point) override;
  BOOL OnDrop(CWnd* pWnd, COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point) override;
  DROPEFFECT OnDropEx(CWnd* pWnd, COleDataObject* pDataObject, DROPEFFECT dropDefault, DROPEFFECT dropList, CPoint point) override;
  void OnDragLeave(CWnd* pWnd) override;
  DROPEFFECT OnDragScroll(CWnd* pWnd, DWORD dwKeyState, CPoint point) override;
  _Success_(return != false) static bool GetFirstPathFromDataObject(_In_ COleDataObject * pDataObject, _Inout_ CString & sFirstPath);
  _NODISCARD static CString GetVolumePathName(_In_ const CString & sPath);

protected:
//Member variables
  CShellTreeCtrl2* m_pTreeCtrl{nullptr};
  CShellListCtrl2* m_pListCtrl{nullptr};
  ATL::CComPtr<IDropTargetHelper> m_DropTargetHelper;
  bool m_bUseDropTargetHelper{false};
};


#endif //#ifndef __SHELLCTRL2DATASOURCE_H__
