/*
Module : ShellCtrl2ScopedRedraw.h
Purpose: Interface for the CShellCtrl2ScopedRedraw class used by the CShellTreeCtrl2 and CShellListCtrl2 classes
Created: PJN / 13-12-2024
History: PJN / 13-12-2024 1. Initial creation

Copyright (c) 2024 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#pragma once

#ifndef __SHELLCTRL2SCOPEDREDRAW_H__
#define __SHELLCTRL2SCOPEDREDRAW_H__

#ifndef SHELLCTRL2SCOPEDREDRAW_EXT_CLASS
#define SHELLCTRL2SCOPEDREDRAW_EXT_CLASS
#endif //#ifndef SHELLCTRL2SCOPEDREDRAW_EXT_CLASS


//////////////////// Classes //////////////////////////////////////////////////

class SHELLCTRL2SCOPEDREDRAW_EXT_CLASS CShellCtrlScopedRedraw
{
public:
//Constructors / Destructors
#pragma warning(suppress: 26429 26447)
  CShellCtrlScopedRedraw(_In_ CWnd* pWnd) : m_pWnd{pWnd}
  {
    pWnd->SetRedraw(FALSE);
  }
  CShellCtrlScopedRedraw() = delete;
  CShellCtrlScopedRedraw(const CShellCtrlScopedRedraw&) = delete;
  CShellCtrlScopedRedraw(CShellCtrlScopedRedraw&&) = delete;
  ~CShellCtrlScopedRedraw()
  {
    //Validate our parameters
#pragma warning(suppress: 26447)
    ASSERT(m_pWnd != nullptr);

#pragma warning(suppress: 26447)
    m_pWnd->SetRedraw(TRUE);
#pragma warning(suppress: 26447)
    m_pWnd->RedrawWindow();
  }

//Methods
  CShellCtrlScopedRedraw& operator=(const CShellCtrlScopedRedraw&) = delete;
  CShellCtrlScopedRedraw& operator=(CShellCtrlScopedRedraw&&) = delete;

//Member variables
  CWnd* m_pWnd{nullptr};
};


#endif //#ifndef __SHELLCTRL2SCOPEDREDRAW_H__
