/*
Module : ShellListCtrl2.h
Purpose: Interface for a CShellListCtrl2 class which is an improved version of the built-in MFC class CMFCShellListCtrl

Copyright (c) 2020 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#pragma once

#ifndef __SHELLLISTCTRL2_H__
#define __SHELLLISTCTRL2_H__

#ifndef SHELLLISTCTRL2_EXT_CLASS
#define SHELLLISTCTRL2_EXT_CLASS
#endif //#ifndef SHELLLISTCTRL2_EXT_CLASS

#ifndef SHELLLISTCTRL2_BASE_CLASS
#define SHELLLISTCTRL2_BASE_CLASS CMFCListCtrl
#endif //#ifndef SHELLLISTCTRL2_BASE_CLASS


//////////////////// Includes /////////////////////////////////////////////////

#ifndef _ATOMIC_
#pragma message("To avoid this message, put atomic in your pre compiled header (normally stdafx.h)")
#include <atomic>
#endif //#ifndef _ATOMIC_

#ifndef __AFXMT_H__
#pragma message("To avoid this message, put afxmt.h in your pre compiled header (normally stdafx.h)")
#include <afxmt.h>
#endif //#ifndef __AFXMT_H__

#include "ShellItemInfo2.h"


//////////////////// Classes //////////////////////////////////////////////////

class CShellTreeCtrl2;

class SHELLLISTCTRL2_EXT_CLASS CShellListCtrl2 : public SHELLLISTCTRL2_BASE_CLASS
{
public:
//Enums
  enum DriveType
  {
    DRIVE_ATTRIBUTE_REMOVABLE = 0x00000001,
    DRIVE_ATTRIBUTE_FIXED = 0x00000002,
    DRIVE_ATTRIBUTE_REMOTE = 0x00000004,
    DRIVE_ATTRIBUTE_CDROM = 0x00000010,
    DRIVE_ATTRIBUTE_RAMDISK = 0x00000020
  };

//Constructors / Destructors
  CShellListCtrl2();
  CShellListCtrl2(const CShellListCtrl2&) = delete;
  CShellListCtrl2(CShellListCtrl2&&) = delete;
  ~CShellListCtrl2() = default; //NOLINT(modernize-use-override)

//Enums
  enum class Columns
  {
    NameColumn = 0,
    DateModifiedColumn = 1,
    TypeColumn = 2,
    SizeColumn = 3
  };

//Methods
  CShellListCtrl2& operator=(const CShellListCtrl2&) = delete;
  CShellListCtrl2& operator=(CShellListCtrl2&&) = delete;
  const ATL::CComPtr<IShellFolder> GetCurrentShellFolder() const noexcept { return m_psfCurFolder; }
  _NODISCARD CPIDL GetCurrentItemIdList() const { return m_pidlCurFQ; }
  _NODISCARD bool IsDesktop() const noexcept { return m_bIsDesktop; }
  bool GetItemPath(_In_ int nItem, _Inout_ CString& sPath) const;
  bool GetItemPIDL(_In_ int nItem, _Inout_ CPIDL& pidl) const;
  bool GetCurrentFolder(_Inout_ CString& sPath) const;
  bool GetCurrentFolderName(_Inout_ CString& sName) const;
  virtual HRESULT Refresh();
  void SetEnumObjectFlags(_In_ SHCONTF types, _In_ bool bRefresh = true);
  _NODISCARD SHCONTF GetEnumObjectFlags() const noexcept { return m_EnumObjectFlags; }
  void SetEnableShellContextMenu(_In_ bool bEnable = true) noexcept { m_bShellContextMenu = bEnable; };
  _NODISCARD bool GetEnableShellContextMenu() const noexcept { return m_bShellContextMenu; };
  void SetEnableNoSelectedItemContextMenu(_In_ bool bEnable = true) noexcept { m_bNoSelectedItemContextMenu = bEnable; };
  _NODISCARD bool GetEnableNoSelectedItemContextMenu() const noexcept { return m_bNoSelectedItemContextMenu; };
  virtual HRESULT DisplayFolder(_In_z_ LPCTSTR lpszPath);
  virtual HRESULT DisplayFolder(_In_ const CPIDL& pidl);
  virtual HRESULT DisplayParentFolder();
  _NODISCARD CShellTreeCtrl2* GetRelatedTree() const;
  void SetRelatedTree(_In_opt_ CShellTreeCtrl2* pShellTree);
  _NODISCARD DWORD_PTR GetItemData(_In_ int nItem) const;
  _NODISCARD void* GetItemDataPtr(_In_ int nItem) const;
  bool SetItemData(_In_ int nItem, _In_ DWORD_PTR dwData);
  bool SetItemDataPtr(_In_ int nItem, _In_opt_ void* pData);
  void SetShowFiles(_In_ bool bShowFiles, _In_ bool bRefresh = true);
  _NODISCARD bool GetShowFiles() const noexcept { return m_EnumObjectFlags & SHCONTF_NONFOLDERS; };
  void SetUsingDifferentIconForSharedFolders(_In_ bool bShowSharedUsingDifferentIcon, _In_ bool bRefresh = true);
  _NODISCARD bool GetUsingDifferentIconForSharedFolders() const noexcept { return m_bShowSharedUsingDifferentIcon; };
  void SetUsingDifferentIconForLinkedItems(_In_ bool bShowLinkedItemsUsingDifferentIcon, _In_ bool bRefresh = true);
  _NODISCARD bool GetUsingDifferentIconForLinkedItems() const noexcept { return m_bShowLinkedItemsUsingDifferentIcon; };
  void SetAllowDoShellDefault(_In_ bool bAllowDoShellDefault) noexcept { m_bAllowDoShellDefault = bAllowDoShellDefault; };
  _NODISCARD bool GetAllowDoShellDefault() const noexcept { return m_bAllowDoShellDefault; };
  void SetDriveHideFlags(_In_ DWORD dwDriveHideFlags, _In_ bool bRefresh = true);
  _NODISCARD DWORD GetDriveHideFlags() const noexcept { return m_dwDriveHideFlags; };
  void SetFileHideFlags(_In_ DWORD dwFileHideFlags, _In_ bool bRefresh = true);
  _NODISCARD DWORD GetFileHideFlags() const noexcept { return m_dwFileHideFlags; };
  void SetFolderHideFlags(_In_ DWORD dwFolderHideFlags, _In_ bool bRefresh = true);
  _NODISCARD DWORD GetFolderHideFlags() const noexcept { return m_dwFolderHideFlags; };
  size_t AddMultipleFilenameMasks(_In_ const CString& sFilenameMasks, _In_ bool bRefresh = true);
  void SetFilenameMask(_In_ size_t nIndex, _In_ const CString& sFilenameMask, _In_ bool bRefresh = true);
  void AddFilenameMask(_In_ const CString& sFilenameMask, _In_ bool bRefresh = true);
  void RemoveFilenameMask(_In_ int nIndex) { m_FilenameMasks.erase(m_FilenameMasks.begin() + nIndex); };
#pragma warning(suppress: 26446)
  _NODISCARD CString GetFilenameMask(_In_ int nIndex) const { return m_FilenameMasks[nIndex]; };
  _NODISCARD CString GetFilenameMasks() const;
  _NODISCARD COLORREF GetCompressedColor() const noexcept { return m_rgbCompressed; };
  void SetCompressedColor(_In_ COLORREF rgbCompressed, _In_ bool bRefresh = true);
  _NODISCARD bool GetUsingDifferentColorForCompressed() const noexcept { return m_bShowCompressedUsingDifferentColor; };
  void SetUsingDifferentColorForCompressed(_In_ bool bShowCompressedUsingDifferentColor, _In_ bool bRefresh = true);
  _NODISCARD COLORREF GetEncryptedColor() const noexcept { return m_rgbEncrypted; };
  void SetEncryptedColor(_In_ COLORREF rgbEncrypted, _In_ bool bRefresh = true);
  _NODISCARD bool GetUsingDifferentColorForEncrypted() const noexcept { return m_bShowEncryptedUsingDifferentColor; };
  void SetUsingDifferentColorForEncrypted(_In_ bool bShowEncryptedUsingDifferentColor, _In_ bool bRefresh = true);
  _NODISCARD bool GetRefreshAfterShellCommand() const noexcept { return m_bRefreshAfterShellCommand; };
  void SetRefreshAfterShellCommand(_In_ bool bRefreshAfterShellCommand) noexcept { m_bRefreshAfterShellCommand = bRefreshAfterShellCommand; };
  _NODISCARD virtual bool IsCompressed(_In_ int nItem);
  _NODISCARD virtual bool IsEncrypted(_In_ int nItem);
  _NODISCARD static bool IsDrive(_In_z_ LPCTSTR pszPath) noexcept;
  virtual bool DoShellDefault(_In_ int nItem, _In_ bool OpenInControlIfPossible = true);
  virtual bool DoShellProperties(_In_ int nItem);
  void SetAutoRefresh(_In_ bool bAutoRefresh);
  _NODISCARD bool GetAutoRefresh() const noexcept { return m_bAutoRefresh; };

protected:
//Member variables
  ATL::CComPtr<IShellFolder> m_psfCurFolder;
  CPIDL m_pidlCurFQ;
  SHCONTF m_EnumObjectFlags;
  HWND m_hwndRelatedTree;
  bool m_bShellContextMenu;
  bool m_bNoSelectedItemContextMenu;
  bool m_bIsDesktop;
  bool m_bNoNotify;
  ATL::CComPtr<IContextMenu2> m_ContextMenu2;
  bool m_bAllowDoShellDefault;
  DWORD m_dwDriveHideFlags;
  DWORD m_dwFileHideFlags;
  DWORD m_dwFolderHideFlags;
  std::vector<CString> m_FilenameMasks;
  COLORREF m_rgbCompressed;
  bool m_bShowCompressedUsingDifferentColor;
  COLORREF m_rgbEncrypted;
  bool m_bShowEncryptedUsingDifferentColor;
  bool m_bShowSharedUsingDifferentIcon;
  bool m_bShowLinkedItemsUsingDifferentIcon;
  bool m_bRefreshAfterShellCommand;
  CWinThread* m_pMonitoringThread;
  CString m_sMonitoringPath;
  CEvent m_MonitoringTerminateEvent;
  std::atomic<bool> m_bAutoRefresh;
  std::atomic<bool> m_bShowFiles;

//Methods
  virtual HRESULT DisplayFolder(_In_opt_ CShellItemInfo2* lpItemInfo);
  virtual bool OnSetColumns();
  virtual CString OnGetItemText(_In_ int nItem, _In_ int nColumn, _In_ CShellItemInfo2* pItem);
  virtual int OnGetItemIcon(_In_ int iItem, _In_ CShellItemInfo2* pItem);
  virtual void OnFormatFileSize(_In_ ULONGLONG nFileSize, _Inout_ CString& sSize);
  virtual void OnFormatFileDate(_In_ const FILETIME& ft, _Inout_ CString& sDate);
  int OnCompareItems(_In_ LPARAM lParam1, _In_ LPARAM lParam2, _In_ int nColumn) override;
  virtual HIMAGELIST GetShellImageList(_In_ bool bLarge);
  virtual HRESULT LockCurrentFolder(_Inout_ ATL::CComPtr<IShellFolder>& pParentFolder, _In_ const CPIDL& pidlRel, _In_ const CPIDL& pidlFQ);
  virtual SFGAOF GetAttributesOfFlags();
  virtual HRESULT EnumObjects(_In_ ATL::CComPtr<IShellFolder>& pParentFolder, _In_ const CPIDL& pidlParent);
  virtual int AddItem(_In_ ATL::CComPtr<IShellFolder>& pParentFolder, _In_ const CPIDL& pidl, _In_ SFGAOF attribs, _In_ const CPIDL& pidlParent);
  virtual bool InitList();
  void PreSubclassWindow() override;
  LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam) override;
  virtual void OnItemAdded(_In_ int nItem);
  virtual void OnShowShellContextMenu(_In_ CPoint point);
  virtual void OnShowNothingSelectedContextMenu();
  virtual bool CanDisplayFile(_In_z_ LPCTSTR pszFilename);
  virtual bool CanDisplayFolder(_In_z_ LPCTSTR pszFolder);
  virtual bool CanDisplayDrive(_In_ const CString& sDrive);
  virtual bool CanDisplay(_In_ const CPIDL& pidlParent, _In_ const CPIDL& pidl);
  virtual void CreateMonitoringThread(_In_z_ LPCTSTR pszPath);
  static UINT _MonitoringThread(_In_ LPVOID pParam);
  virtual UINT MonitoringThread();
  virtual void KillMonitoringThread();
  virtual bool CanHandleChangeNotifications(_In_z_ LPCTSTR pszPath);
  BOOL PreTranslateMessage(MSG* pMsg) override;

//Message handlers
  afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
  afx_msg void OnDeleteItem(NMHDR* pNMHDR, LRESULT* pResult);
  afx_msg void OnDblClk(NMHDR* pNMHDR, LRESULT* pResult);
  afx_msg void OnReturn(NMHDR* pNMHDR, LRESULT* pResult);
  afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
  afx_msg void OnDestroy();
  afx_msg void OnCustomDraw(NMHDR* pNMHDR, LRESULT* pResult);
  afx_msg LRESULT OnNotificationChange(WPARAM wParam, LPARAM lParam);

  friend class CShellTreeCtrl2;

  DECLARE_MESSAGE_MAP()
  DECLARE_DYNCREATE(CShellListCtrl2)
};


#endif //#ifndef __SHELLLISTCTRL2_H__
