/*
Module : ShellListCtrl2.h
Purpose: Interface for a CShellListCtrl2 class which is an improved version of the built-in MFC class CMFCShellListCtrl

Copyright (c) 2020 - 2024 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#pragma once

#if _MSVC_LANG < 201703
#error CShellListCtrl2 requires a minimum C++ language standard of /std:c++17
#endif //#if _MSVC_LANG < 201703

#ifndef __SHELLLISTCTRL2_H__
#define __SHELLLISTCTRL2_H__

#ifndef SHELLLISTCTRL2_EXT_CLASS
#define SHELLLISTCTRL2_EXT_CLASS
#endif //#ifndef SHELLLISTCTRL2_EXT_CLASS

#ifndef SHELLLISTCTRL2_BASE_CLASS
#define SHELLLISTCTRL2_BASE_CLASS CMFCListCtrl
#endif //#ifndef SHELLLISTCTRL2_BASE_CLASS


//////////////////// Includes /////////////////////////////////////////////////

#ifndef _ATOMIC_
#pragma message("To avoid this message, put atomic in your pre compiled header (normally stdafx.h)")
#include <atomic>
#endif //#ifndef _ATOMIC_

#ifndef __AFXMT_H__
#pragma message("To avoid this message, put afxmt.h in your pre compiled header (normally stdafx.h)")
#include <afxmt.h>
#endif //#ifndef __AFXMT_H__

#include "ShellItemInfo2.h"
#include "ShellCtrl2DataSource.h"
#include "ShellCtrl2DropTarget.h"


//////////////////// Defines //////////////////////////////////////////////////

constexpr const TCHAR* CSHELLLISTCTRL2_WM_SHELLLISTCTRL2_AFTER_SHELL_COMMAND{_T("WM_SHELLLISTCTRL2_AFTER_SHELL_COMMAND")};
constexpr const TCHAR* CSHELLLISTCTRL2_WM_SHELLLISTCTRL2_NOTIFICATION_CHANGE{_T("WM_SHELLLISTCTRL2_NOTIFICATION_CHANGE")};
constexpr const TCHAR* CSHELLLISTCTRL2_WM_SHELLLISTCTRL2_AFTER_OLEDROP{_T("WM_SHELLLISTCTRL2_AFTER_OLEDROP")};


//////////////////// Classes //////////////////////////////////////////////////

class CShellTreeCtrl2;

class SHELLLISTCTRL2_EXT_CLASS CShellListCtrl2 : public SHELLLISTCTRL2_BASE_CLASS
{
public:
//Enums
  enum DriveType
  {
    DRIVE_ATTRIBUTE_REMOVABLE = 0x00000001,
    DRIVE_ATTRIBUTE_FIXED = 0x00000002,
    DRIVE_ATTRIBUTE_REMOTE = 0x00000004,
    DRIVE_ATTRIBUTE_CDROM = 0x00000010,
    DRIVE_ATTRIBUTE_RAMDISK = 0x00000020
  };

//Constructors / Destructors
  CShellListCtrl2();
  CShellListCtrl2(const CShellListCtrl2&) = delete;
  CShellListCtrl2(CShellListCtrl2&&) = delete;
  ~CShellListCtrl2(); //NOLINT(modernize-use-override)

//Enums
  enum class Columns
  {
    NameColumn = 0,
    DateModifiedColumn = 1,
    TypeColumn = 2,
    SizeColumn = 3
  };

//Methods
  CShellListCtrl2& operator=(const CShellListCtrl2&) = delete;
  CShellListCtrl2& operator=(CShellListCtrl2&&) = delete;
  const ATL::CComPtr<IShellFolder> GetCurrentShellFolder() const noexcept { return m_psfCurFolder; }
  _NODISCARD CPIDL GetCurrentItemIdList() const { return m_pidlCurFQ; }
  _NODISCARD bool IsDesktop() const noexcept { return m_bIsDesktop; }
  _Success_(return != false) bool GetItemPath(_In_ int nItem, _Inout_ CString& sPath) const;
  _Success_(return != false) bool GetItemPIDL(_In_ int nItem, _Inout_ CPIDL& pidl) const;
  _Success_(return != false) bool GetCurrentFolder(_Inout_ CString& sPath) const;
  _Success_(return != false) bool GetCurrentFolderName(_Inout_ CString& sName) const;
  virtual HRESULT Refresh();
  void SetEnumObjectFlags(_In_ SHCONTF types, _In_ bool bRefresh = true);
  _NODISCARD SHCONTF GetEnumObjectFlags() const noexcept { return m_EnumObjectFlags; }
  void SetEnableShellContextMenu(_In_ bool bEnable = true) noexcept { m_bShellContextMenu = bEnable; };
  _NODISCARD _Success_(return != false) bool GetEnableShellContextMenu() const noexcept { return m_bShellContextMenu; };
  void SetEnableNoSelectedItemContextMenu(_In_ bool bEnable = true) noexcept { m_bNoSelectedItemContextMenu = bEnable; };
  _NODISCARD _Success_(return != false) bool GetEnableNoSelectedItemContextMenu() const noexcept { return m_bNoSelectedItemContextMenu; };
  virtual HRESULT DisplayFolder(_In_z_ LPCTSTR lpszPath);
  virtual HRESULT DisplayFolder(_In_ const CPIDL& pidl);
  virtual HRESULT DisplayParentFolder();
  _NODISCARD CShellTreeCtrl2* GetRelatedTree() const;
  void SetRelatedTree(_In_opt_ CShellTreeCtrl2* pShellTree);
  _NODISCARD DWORD_PTR GetItemData(_In_ int nItem) const;
  _NODISCARD void* GetItemDataPtr(_In_ int nItem) const;
  _Success_(return != false) bool SetItemData(_In_ int nItem, _In_ DWORD_PTR dwData);
  _Success_(return != false) bool SetItemDataPtr(_In_ int nItem, _In_opt_ void* pData);
  void SetShowFiles(_In_ bool bShowFiles, _In_ bool bRefresh = true);
  _NODISCARD _Success_(return != false) bool GetShowFiles() const noexcept { return m_EnumObjectFlags & SHCONTF_NONFOLDERS; };
  void SetUsingDifferentIconForSharedFolders(_In_ bool bShowSharedUsingDifferentIcon, _In_ bool bRefresh = true);
  _NODISCARD _Success_(return != false) bool GetUsingDifferentIconForSharedFolders() const noexcept { return m_bShowSharedUsingDifferentIcon; };
  void SetUsingDifferentIconForLinkedItems(_In_ bool bShowLinkedItemsUsingDifferentIcon, _In_ bool bRefresh = true);
  _NODISCARD _Success_(return != false) bool GetUsingDifferentIconForLinkedItems() const noexcept { return m_bShowLinkedItemsUsingDifferentIcon; };
  void SetAllowDoShellDefault(_In_ bool bAllowDoShellDefault) noexcept { m_bAllowDoShellDefault = bAllowDoShellDefault; };
  _NODISCARD _Success_(return != false) bool GetAllowDoShellDefault() const noexcept { return m_bAllowDoShellDefault; };
  void SetDriveHideFlags(_In_ DWORD dwDriveHideFlags, _In_ bool bRefresh = true);
  _NODISCARD DWORD GetDriveHideFlags() const noexcept { return m_dwDriveHideFlags; };
  void SetFileHideFlags(_In_ DWORD dwFileHideFlags, _In_ bool bRefresh = true);
  _NODISCARD DWORD GetFileHideFlags() const noexcept { return m_dwFileHideFlags; };
  void SetFolderHideFlags(_In_ DWORD dwFolderHideFlags, _In_ bool bRefresh = true);
  _NODISCARD DWORD GetFolderHideFlags() const noexcept { return m_dwFolderHideFlags; };
  size_t AddMultipleFilenameMasks(_In_ const CString& sFilenameMasks, _In_ bool bRefresh = true);
  void SetFilenameMask(_In_ size_t nIndex, _In_ const CString& sFilenameMask, _In_ bool bRefresh = true);
  void AddFilenameMask(_In_ const CString& sFilenameMask, _In_ bool bRefresh = true);
  void RemoveFilenameMask(_In_ int nIndex) { m_FilenameMasks.erase(m_FilenameMasks.begin() + nIndex); };
#pragma warning(suppress: 26446)
  _NODISCARD CString GetFilenameMask(_In_ int nIndex) const { return m_FilenameMasks[nIndex]; };
  _NODISCARD CString GetFilenameMasks() const;
  _NODISCARD COLORREF GetCompressedColor() const noexcept { return m_rgbCompressed; };
  void SetCompressedColor(_In_ COLORREF rgbCompressed, _In_ bool bRefresh = true);
  _NODISCARD _Success_(return != false) bool GetUsingDifferentColorForCompressed() const noexcept { return m_bShowCompressedUsingDifferentColor; };
  void SetUsingDifferentColorForCompressed(_In_ bool bShowCompressedUsingDifferentColor, _In_ bool bRefresh = true);
  _NODISCARD COLORREF GetEncryptedColor() const noexcept { return m_rgbEncrypted; };
  void SetEncryptedColor(_In_ COLORREF rgbEncrypted, _In_ bool bRefresh = true);
  _NODISCARD _Success_(return != false) bool GetUsingDifferentColorForEncrypted() const noexcept { return m_bShowEncryptedUsingDifferentColor; };
  void SetUsingDifferentColorForEncrypted(_In_ bool bShowEncryptedUsingDifferentColor, _In_ bool bRefresh = true);
  _NODISCARD _Success_(return != false) bool GetRefreshAfterShellCommand() const noexcept { return m_bRefreshAfterShellCommand; };
  void SetRefreshAfterShellCommand(_In_ bool bRefreshAfterShellCommand) noexcept { m_bRefreshAfterShellCommand = bRefreshAfterShellCommand; };
  _NODISCARD _Success_(return != false) virtual bool IsCompressed(_In_ int nItem);
  _NODISCARD _Success_(return != false) virtual bool IsEncrypted(_In_ int nItem);
  _NODISCARD _Success_(return != false) static bool IsDrive(_In_z_ LPCTSTR pszPath) noexcept;
  _Success_(return != false) virtual bool DoShellDefault(_In_ int nItem, _In_ bool OpenInControlIfPossible = true);
  _Success_(return != false) virtual bool DoShellProperties(_In_ int nItem);
  void SetAutoRefresh(_In_ bool bAutoRefresh);
  _NODISCARD _Success_(return != false) bool GetAutoRefresh() const noexcept { return m_bAutoRefresh; };
  _NODISCARD _Success_(return != false) virtual bool IsDropSource(_In_ int nItem);
  void SetAllowDragDrop(_In_ bool bAllowDragDrop);
  _NODISCARD bool GetAllowDragDrop() const noexcept { return m_bAllowDragDrop; };

protected:
//Member variables
  ATL::CComPtr<IShellFolder> m_psfCurFolder;
  CPIDL m_pidlCurFQ;
  SHCONTF m_EnumObjectFlags;
  HWND m_hRelatedTree;
  bool m_bShellContextMenu;
  bool m_bNoSelectedItemContextMenu;
  bool m_bIsDesktop;
  bool m_bNoNotify;
  bool m_bAllowDragDrop;
  ATL::CComPtr<IContextMenu2> m_ContextMenu2;
  bool m_bAllowDoShellDefault;
  DWORD m_dwDriveHideFlags;
  DWORD m_dwFileHideFlags;
  DWORD m_dwFolderHideFlags;
  std::vector<CString> m_FilenameMasks;
  COLORREF m_rgbCompressed;
  bool m_bShowCompressedUsingDifferentColor;
  COLORREF m_rgbEncrypted;
  bool m_bShowEncryptedUsingDifferentColor;
  bool m_bShowSharedUsingDifferentIcon;
  bool m_bShowLinkedItemsUsingDifferentIcon;
  bool m_bRefreshAfterShellCommand;
  CWinThread* m_pMonitoringThread;
  CString m_sMonitoringPath;
  CEvent m_MonitoringTerminateEvent;
  std::atomic<bool> m_bAutoRefresh;
  std::atomic<bool> m_bShowFiles;
  CShellCtrl2DropTarget* m_pOLEDropTarget;
  bool m_bCanAcceptData;
  bool m_bLeftMouseDrag;
  int m_nItemDrop;
  ULONGLONG m_nLastMovementTime;
  ULONGLONG m_nLastScrollTime;
  CPoint m_lastMouseLoc;
  UINT m_nMouseHoverTime;
  bool m_bDropContainedDirectories;
  bool m_bDropContainedFiles;
  int m_nDropTarget;
  bool m_bDropTargetWasSelected;

//Methods
  virtual HRESULT DisplayFolder(_In_opt_ CShellItemInfo2* lpItemInfo);
  _Success_(return != false) virtual bool OnSetColumns();
  _NODISCARD virtual CString OnGetItemText(_In_ int nItem, _In_ int nColumn, _In_ CShellItemInfo2* pItem);
  _NODISCARD virtual int OnGetItemIcon(_In_ int iItem, _In_ CShellItemInfo2* pItem);
  virtual void OnFormatFileSize(_In_ ULONGLONG nFileSize, _Inout_ CString& sSize);
  virtual void OnFormatFileDate(_In_ const FILETIME& ft, _Inout_ CString& sDate);
  _NODISCARD int OnCompareItems(_In_ LPARAM lParam1, _In_ LPARAM lParam2, _In_ int nColumn) override;
  _NODISCARD virtual HIMAGELIST GetShellImageList(_In_ bool bLarge);
  virtual HRESULT LockCurrentFolder(_Inout_ ATL::CComPtr<IShellFolder>& pParentFolder, _In_ const CPIDL& pidlRel, _In_ const CPIDL& pidlFQ);
  _NODISCARD virtual SFGAOF GetAttributesOfFlags();
  virtual HRESULT EnumObjects(_In_ ATL::CComPtr<IShellFolder>& pParentFolder, _In_ const CPIDL& pidlParent);
  virtual int AddItem(_In_ ATL::CComPtr<IShellFolder>& pParentFolder, _In_ const CPIDL& pidl, _In_ SFGAOF attribs, _In_ const CPIDL& pidlParent);
  _Success_(return != false) virtual bool InitList();
  void PreSubclassWindow() override;
  LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam) override;
  virtual void OnItemAdded(_In_ int nItem);
  virtual void OnShowShellContextMenu(_In_ CPoint point);
  virtual void OnShowNothingSelectedContextMenu();
  _Success_(return != false) virtual bool CanDisplayFile(_In_z_ LPCTSTR pszFilename);
  _Success_(return != false) virtual bool CanDisplayFolder(_In_z_ LPCTSTR pszFolder);
  _Success_(return != false) virtual bool CanDisplayDrive(_In_ const CString& sDrive);
  _Success_(return != false) virtual bool CanDisplay(_In_ const CPIDL& pidlParent, _In_ const CPIDL& pidl);
  virtual void CreateMonitoringThread(_In_z_ LPCTSTR pszPath);
  static UINT _MonitoringThread(_In_ LPVOID pParam);
  virtual UINT MonitoringThread();
  virtual void KillMonitoringThread();
  _Success_(return != false) virtual bool CanHandleChangeNotifications(_In_z_ LPCTSTR pszPath);
  BOOL PreTranslateMessage(MSG* pMsg) override;
  _Success_(return != false) virtual bool CreateDragDropComponents();
  virtual DROPEFFECT OnDragEnter(_In_ COleDataObject* pDataObject, _In_ DWORD dwKeyState, _In_ CPoint point);
  virtual DROPEFFECT OnDragOver(_In_ COleDataObject* pDataObject, _In_ DWORD dwKeyState, _In_ CPoint point);
  _Success_(return != FALSE) virtual BOOL OnDrop(_In_ COleDataObject* pDataObject, _In_ DROPEFFECT dropEffect, _In_ CPoint point);
  virtual DROPEFFECT OnDropEx(_In_ COleDataObject* pDataObject, _In_ DROPEFFECT dropDefault, _In_ DROPEFFECT dropList, _In_ CPoint point);
  virtual void OnDragLeave();
  virtual DROPEFFECT OnDragScroll(_In_ DWORD dwKeyState, _In_ CPoint point);
  _NODISCARD virtual _Success_(return != false) bool IsOverValidDropItem(_In_ const CPoint& point);
  virtual void ResetHoverTimer(_In_ ULONGLONG dwTickCount);
  virtual DROPEFFECT PerformRightClickDragDrop(_In_ int nItem, _In_ DWORD dwDropDefault, _In_ DROPEFFECT dwDropList, _In_ HDROP hDrop, _In_ const CString& sToPath);
  virtual HRESULT PerformDragDrop(_In_ int nItem, _In_ DWORD dwDropEffect, _In_ HDROP hDrop, _In_ const CString& sToPath);
  virtual void SelectDropTarget(_In_ int nItem);

//Message handlers
  afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
  afx_msg void OnDeleteItem(NMHDR* pNMHDR, LRESULT* pResult);
  afx_msg void OnDblClk(NMHDR* pNMHDR, LRESULT* pResult);
  afx_msg void OnReturn(NMHDR* pNMHDR, LRESULT* pResult);
  afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
  afx_msg void OnDestroy();
  afx_msg void OnCustomDraw(NMHDR* pNMHDR, LRESULT* pResult);
  afx_msg LRESULT OnNotificationChange(WPARAM wParam, LPARAM lParam);
  afx_msg void OnBeginDrag(NMHDR* pNMHDR, LRESULT* pResult);
  afx_msg void OnBeginRDrag(NMHDR* pNMHDR, LRESULT* pResult);

  friend class CShellTreeCtrl2;
  friend class CShellCtrl2DropTarget;

  DECLARE_MESSAGE_MAP()
  DECLARE_DYNCREATE(CShellListCtrl2)
};


#endif //#ifndef __SHELLLISTCTRL2_H__
