/*
Module : ShellTreeCtrl2.cpp
Purpose: Implementation for the CShellTreeCtrl2 class
Created: PJN / 25-07-2020
History: PJN / 25-07-2020 1. Initial creation
         PJN / 23-04-2022 1. Updated copyright details.
                          2. Updated the code to use C++ uniform initialization for all variable
                          declarations.

Copyright (c) 2020 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Includes /////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"
#include "ShellTreeCtrl2.h"
#include "ShellListCtrl2.h"


//////////////////// Defines / Locals /////////////////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#endif //#ifdef _DEBUG

#pragma warning(suppress: 26426)
const UINT WM_SHELLTREECTRL2_AFTER_SHELL_COMMAND = ::RegisterWindowMessage(_T("WM_SHELLTREECTRL2_AFTER_SHELL_COMMAND"));
#pragma warning(suppress: 26426)
const UINT WM_SHELLTREECTRL2_NOTIFICATION_CHANGE = ::RegisterWindowMessage(_T("WM_SHELLTREECTRL2_NOTIFICATION_CHANGE"));


//////////////////// Implementation ///////////////////////////////////////////

#pragma warning(suppress: 26409 26440 26433 26477)
IMPLEMENT_DYNCREATE(CShellTreeCtrl2, SHELLTREECTRL2_BASE_CLASS)

#pragma warning(suppress: 26433 26440)
BEGIN_MESSAGE_MAP(CShellTreeCtrl2, SHELLTREECTRL2_BASE_CLASS) //NOLINT(modernize-avoid-c-arrays)
  ON_WM_CREATE()
  ON_WM_DESTROY()
  ON_WM_CONTEXTMENU()
#pragma warning(suppress: 26454)
  ON_NOTIFY_REFLECT(NM_CUSTOMDRAW, &CShellTreeCtrl2::OnCustomDraw)
#pragma warning(suppress: 26454)
  ON_NOTIFY_REFLECT(TVN_SELCHANGED, &CShellTreeCtrl2::OnSelChanged)
  ON_WM_RBUTTONDOWN()
#pragma warning(suppress: 26454)
  ON_NOTIFY_REFLECT(TVN_ITEMEXPANDING, &CShellTreeCtrl2::OnItemExpanding)
#pragma warning(suppress: 26454)
  ON_NOTIFY_REFLECT(TVN_DELETEITEM, &CShellTreeCtrl2::OnDeleteItem)
#pragma warning(suppress: 26454)
  ON_NOTIFY_REFLECT(NM_DBLCLK, &CShellTreeCtrl2::OnDblClk)
  ON_REGISTERED_MESSAGE(WM_SHELLTREECTRL2_NOTIFICATION_CHANGE, &CShellTreeCtrl2::OnNotificationChange)
END_MESSAGE_MAP()

#pragma warning(suppress: 26455 26495)
CShellTreeCtrl2::CShellTreeCtrl2() : SHELLTREECTRL2_BASE_CLASS{},
                                     m_bAllowDoShellDefault{false},
                                     m_dwDriveHideFlags{0},
                                     m_dwFileHideFlags{FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM | FILE_ATTRIBUTE_OFFLINE | FILE_ATTRIBUTE_TEMPORARY},
                                     m_dwFolderHideFlags{FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM | FILE_ATTRIBUTE_OFFLINE | FILE_ATTRIBUTE_TEMPORARY},
                                     m_rgbCompressed{RGB(0, 0, 255)},
                                     m_bShowCompressedUsingDifferentColor{true},
                                     m_rgbEncrypted{RGB(98, 146, 13)},
                                     m_bShowEncryptedUsingDifferentColor{true},
                                     m_nMaxHistory{20},
                                     m_bUpdatingHistorySelection{false},
                                     m_bShowSharedUsingDifferentIcon{true},
                                     m_bShowLinkedItemsUsingDifferentIcon{true},
                                     m_bKeepChildrenOnCollapse{false},
                                     m_bShowRootedFolder{true},
                                     m_bContextMenu{true},
                                     m_bNoNotify{false},
                                     m_hwndRelatedList{nullptr},
                                     m_EnumObjectFlags{SHCONTF_FOLDERS | SHCONTF_INCLUDEHIDDEN | SHCONTF_INCLUDESUPERHIDDEN},
                                     m_bRefreshAfterShellCommand{true},
                                     m_pMonitoringThread{nullptr},
                                     m_bAutoRefresh{false},
                                     m_bShowFiles{false}
{
  m_FilenameMasks.emplace_back(_T("*.*"));
}

void CShellTreeCtrl2::SetShowFiles(_In_ bool bShowFiles, _In_ bool bRefresh)
{
  if (bShowFiles != m_bShowFiles)
  {
    m_bShowFiles = bShowFiles;
    if (bShowFiles)
      m_EnumObjectFlags |= SHCONTF_NONFOLDERS;
    else
      m_EnumObjectFlags &= ~SHCONTF_NONFOLDERS;
    if (bRefresh && (GetSafeHwnd() != nullptr))
      Refresh();
  }
}

void CShellTreeCtrl2::SetRootFolder(_In_ const CPIDL& root)
{
  m_RootFolder = root;
  Refresh();
}

HRESULT CShellTreeCtrl2::SetRootFolder(_In_z_ LPCTSTR pszRoot)
{
  const HRESULT hr{m_RootFolder.FromPath(pszRoot)};
  if (FAILED(hr))
    return hr;
  Refresh();
  return S_OK;
}

bool CShellTreeCtrl2::DoShellDefault(_In_ HTREEITEM hItem)
{
  if (!m_bAllowDoShellDefault)
    return false;

  TVITEM tvItem{};
  tvItem.hItem = hItem;
  tvItem.mask = TVIF_HANDLE | TVIF_PARAM;
  if (!GetItem(&tvItem))
  {
    ASSERT(FALSE);
    return false;
  }
#pragma warning(suppress: 26490)
  const CShellItemInfo2* pItem{reinterpret_cast<CShellItemInfo2*>(tvItem.lParam)};
  if ((pItem == nullptr) || (pItem->m_pParentFolder == nullptr) || (!pItem->m_pidlRel.operator bool()))
  {
    ASSERT(FALSE);
    return false;
  }
  const CWnd* pParent{GetParent()};
  if (pParent == nullptr)
  {
    ASSERT(FALSE);
    return false;
  }

  //Invoke the default menu command
  ATL::CComPtr<IContextMenu> pcm;
#pragma warning(suppress: 26490)
  HRESULT hr{pItem->m_pParentFolder->GetUIObjectOf(GetSafeHwnd(), 1, pItem->m_pidlRel, IID_IContextMenu, nullptr, reinterpret_cast<void**>(&pcm))};
  if (FAILED(hr))
    return false;
  CMenu popup;
  if (!popup.CreatePopupMenu())
    return false;
  hr = pcm->QueryContextMenu(popup, 0, 1, 0x7FFFF, CMF_DEFAULTONLY | CMF_EXPLORE);
  if (FAILED(hr))
    return false;
  const UINT nIDCmd{popup.GetDefaultItem(0, FALSE)};
#pragma warning(suppress: 26472)
  if ((nIDCmd == 0) || (nIDCmd == static_cast<UINT>(-1)))
    return false;
  CMINVOKECOMMANDINFO cmi{};
  cmi.cbSize = sizeof(CMINVOKECOMMANDINFO);
  cmi.hwnd = pParent->GetSafeHwnd();
#pragma warning(suppress: 26472 26490)
  cmi.lpVerb = reinterpret_cast<LPCSTR>(static_cast<INT_PTR>(nIDCmd - 1));
  cmi.nShow = SW_SHOWNORMAL;
  hr = pcm->InvokeCommand(&cmi);
  if (SUCCEEDED(hr))
#pragma warning(suppress: 26472)
    pParent->SendMessage(WM_SHELLTREECTRL2_AFTER_SHELL_COMMAND, static_cast<WPARAM>(nIDCmd));
  SetFocus();
  return SUCCEEDED(hr);
}

bool CShellTreeCtrl2::DoShellProperties(_In_ HTREEITEM hItem)
{
  TVITEM tvItem{};
  tvItem.hItem = hItem;
  tvItem.mask = TVIF_HANDLE | TVIF_PARAM;
  if (!GetItem(&tvItem))
  {
    ASSERT(FALSE);
    return false;
  }
#pragma warning(suppress: 26490)
  const CShellItemInfo2* pItem{reinterpret_cast<CShellItemInfo2*>(tvItem.lParam)};
  if (pItem == nullptr)
  {
    ASSERT(FALSE);
    return false;
  }
  const CWnd* pParent{GetParent()};
  if (pParent == nullptr)
  {
    ASSERT(FALSE);
    return false;
  }
#pragma warning(suppress: 26476)
  SHELLEXECUTEINFO sei{};
  sei.cbSize = sizeof(sei);
  sei.hwnd = pParent->GetSafeHwnd();
  sei.nShow = SW_SHOW;
#pragma warning(suppress: 26489)
  sei.lpIDList = pItem->m_pidlFQ.m_pPIDL;
  sei.lpVerb = _T("properties");
  sei.fMask = SEE_MASK_INVOKEIDLIST | SEE_MASK_IDLIST;
#pragma warning(suppress: 26486)
  return ShellExecuteEx(&sei);
}

void CShellTreeCtrl2::SetDriveHideFlags(_In_ DWORD dwDriveHideFlags, _In_ bool bRefresh)
{
  m_dwDriveHideFlags = dwDriveHideFlags;
  if (bRefresh && (GetSafeHwnd() != nullptr))
    Refresh();
}

void CShellTreeCtrl2::SetFileHideFlags(_In_ DWORD dwFileHideFlags, _In_ bool bRefresh)
{
  m_dwFileHideFlags = dwFileHideFlags;
  if (bRefresh && (GetSafeHwnd() != nullptr))
    Refresh();
}

void CShellTreeCtrl2::SetFolderHideFlags(_In_ DWORD dwFolderHideFlags, _In_ bool bRefresh)
{
  m_dwFolderHideFlags = dwFolderHideFlags;
  if (bRefresh && (GetSafeHwnd() != nullptr))
    Refresh();
}

size_t CShellTreeCtrl2::AddMultipleFilenameMasks(_In_ const CString& sFilenameMasks, _In_ bool bRefresh)
{
  //Empty out the array
  m_FilenameMasks.clear();

  int nCurPos{0};
  CString sToken{sFilenameMasks.Tokenize(_T(",;"), nCurPos)};
  while (sToken != _T("")) //NOLINT(clang-analyzer-core.CallAndMessage)
  {
    sToken.Trim();
    m_FilenameMasks.push_back(sToken);
    sToken = sFilenameMasks.Tokenize(_T(",;"), nCurPos);
  }

  if (bRefresh && (GetSafeHwnd() != nullptr))
    Refresh();

  //Return the number of recipients parsed
  return m_FilenameMasks.size();
}

CString CShellTreeCtrl2::GetFilenameMasks() const
{
  //What will be the return value from this function
  CString sMasks;

  const auto nSize{m_FilenameMasks.size()};
  for (size_t i{0}; i<nSize; i++)
  {
#pragma warning(suppress: 26446)
    sMasks += m_FilenameMasks[i];
    if (i < (nSize-1))
      sMasks += _T(",");
  }
  return sMasks;
}

void CShellTreeCtrl2::SetFilenameMask(_In_ size_t nIndex, _In_ const CString& sFileNameMask, _In_ bool bRefresh)
{
#pragma warning(suppress: 26446)
  m_FilenameMasks[nIndex] = sFileNameMask;

  if (bRefresh && (GetSafeHwnd() != nullptr))
    Refresh();
}

void CShellTreeCtrl2::AddFilenameMask(_In_ const CString& sFileNameMask, _In_ bool bRefresh)
{
  m_FilenameMasks.push_back(sFileNameMask);
  if (bRefresh && (GetSafeHwnd() != nullptr))
    Refresh();
}

void CShellTreeCtrl2::SetCompressedColor(_In_ COLORREF rgbCompressed, _In_ bool bRefresh)
{
  m_rgbCompressed = rgbCompressed;
  if (bRefresh && (GetSafeHwnd() != nullptr))
    Refresh();
}

void CShellTreeCtrl2::SetEncryptedColor(_In_ COLORREF rgbEncrypted, _In_ bool bRefresh)
{
  m_rgbEncrypted = rgbEncrypted;
  if (bRefresh && (GetSafeHwnd() != nullptr))
    Refresh();
}

void CShellTreeCtrl2::SetUsingDifferentColorForCompressed(_In_ bool bShowCompressedUsingDifferentColor, _In_ bool bRefresh)
{
  m_bShowCompressedUsingDifferentColor = bShowCompressedUsingDifferentColor;
  if (bRefresh && (GetSafeHwnd() != nullptr))
    Refresh();
}

void CShellTreeCtrl2::SetUsingDifferentIconForSharedFolders(_In_ bool bShowSharedUsingDifferentIcon, _In_ bool bRefresh)
{
  m_bShowSharedUsingDifferentIcon = bShowSharedUsingDifferentIcon;
  if (bRefresh && (GetSafeHwnd() != nullptr))
    Refresh();
}

void CShellTreeCtrl2::SetUsingDifferentIconForLinkedItems(_In_ bool bShowLinkedItemsUsingDifferentIcon, _In_ bool bRefresh)
{
  m_bShowLinkedItemsUsingDifferentIcon = bShowLinkedItemsUsingDifferentIcon;
  if (bRefresh && (GetSafeHwnd() != nullptr))
    Refresh();
}

void CShellTreeCtrl2::SetUsingDifferentColorForEncrypted(_In_ bool bShowEncryptedUsingDifferentColor, _In_ bool bRefresh)
{
  m_bShowEncryptedUsingDifferentColor = bShowEncryptedUsingDifferentColor;
  if (bRefresh && (GetSafeHwnd() != nullptr))
    Refresh();
};

void CShellTreeCtrl2::SetShowRootedFolder(_In_ bool bShowRootedFolder, _In_ bool bRefresh)
{
  m_bShowRootedFolder = bShowRootedFolder;
  if (bRefresh && (GetSafeHwnd() != nullptr))
    Refresh();
}

bool CShellTreeCtrl2::CanDisplayFile(_In_z_ LPCTSTR pszFilename)
{
  CString sDrive;
  CString sDir;
  _tsplitpath_s(pszFilename, sDrive.GetBuffer(_MAX_DRIVE), _MAX_DRIVE, sDir.GetBuffer(_MAX_DIR), _MAX_DIR, nullptr, 0, nullptr, 0);
  sDir.ReleaseBuffer();
  sDrive.ReleaseBuffer();

  //Now check to see if there is any files of the specified file mask
  for (const auto& sMask : m_FilenameMasks)
  {
    CString sFname;
    CString sExt;
    _tsplitpath_s(sMask, nullptr, 0, nullptr, 0, sFname.GetBuffer(_MAX_FNAME), _MAX_FNAME, sExt.GetBuffer(_MAX_EXT), _MAX_EXT);
    sExt.ReleaseBuffer();
    CString sSearch;
    _tmakepath_s(sSearch.GetBuffer(_MAX_PATH), _MAX_PATH, sDrive, sDir, sFname, sExt);
    sSearch.ReleaseBuffer();

    CFileFind find;
    BOOL bFind{find.FindFile(sSearch)};
    while (bFind)
    {
      bFind = find.FindNextFile();
      if ((find.GetFilePath().CompareNoCase(pszFilename) == 0) && !find.MatchesMask(m_dwFileHideFlags))
        return true;
    }
  }
  return false;
}

#pragma warning(suppress: 26440)
bool CShellTreeCtrl2::CanDisplayFolder(_In_z_ LPCTSTR pszFolder)
{
  //Derived classes can decide dynamically whether or not a certain folder is to be displayed. CShellTreeCtrl2 by default
  //displays all folders which do not have attributes as set in m_dwFolderHideFlags

  const DWORD dwAttributes{GetFileAttributes(pszFolder)};
  return ((dwAttributes != INVALID_FILE_ATTRIBUTES) && ((dwAttributes & m_dwFolderHideFlags) == 0));
}

#pragma warning(suppress: 26440)
bool CShellTreeCtrl2::CanDisplayDrive(_In_ const CString& sDrive)
{
  //Derived classes can decide dynamically whether or not a certain drive is to be displayed.
  //CShellTreeCtrl2 by default displays all drives which do not have attributes as set in m_dwDriveHideFlags

  //check if this drive is one of the types to hide
  bool bDisplay{true};
  const UINT nDrive{GetDriveType(sDrive)};
  switch (nDrive)
  {
    case DRIVE_REMOVABLE:
    {
      if (m_dwDriveHideFlags & static_cast<DWORD>(DriveType::DRIVE_ATTRIBUTE_REMOVABLE))
        bDisplay = false;
      break;
    }
    case DRIVE_FIXED:
    {
      if (m_dwDriveHideFlags & static_cast<DWORD>(DriveType::DRIVE_ATTRIBUTE_FIXED))
        bDisplay = false;
      break;
    }
    case DRIVE_REMOTE:
    {
      if (m_dwDriveHideFlags & static_cast<DWORD>(DriveType::DRIVE_ATTRIBUTE_REMOTE))
        bDisplay = false;
      break;
    }
    case DRIVE_CDROM:
    {
      if (m_dwDriveHideFlags & static_cast<DWORD>(DriveType::DRIVE_ATTRIBUTE_CDROM))
        bDisplay = false;
      break;
    }
    case DRIVE_RAMDISK:
    {
      if (m_dwDriveHideFlags & static_cast<DWORD>(DriveType::DRIVE_ATTRIBUTE_RAMDISK))
        bDisplay = false;
      break;
    }
    default:
    {
      break;
    }
  }

  return bDisplay;
}

bool CShellTreeCtrl2::CanDisplay(_In_ const CPIDL& pidlParent, _In_ const CPIDL& pidl)
{
  //Derived classes can decide dynamically whether or not a certain PIDL is to be displayed.
  //The default implementation checks via CanDisplayFolder for file system folders,
  //CanDisplayFile for file system files (non-folders) and CanDisplayDrive for drive letters
  //and if not any of the above, then allows the display

  bool bAddItem{false};
  CPIDL fqPidl{pidlParent + pidl};
  CString sPath;
  if (fqPidl.ToPath(sPath))
  {
    const DWORD dwAttributes{GetFileAttributes(sPath)};
    const bool bIsFile{(dwAttributes != INVALID_FILE_ATTRIBUTES) && ((dwAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)};
    const bool bIsFolder{(dwAttributes != INVALID_FILE_ATTRIBUTES) && (dwAttributes & FILE_ATTRIBUTE_DIRECTORY)};
#pragma warning(suppress: 26446)
    if ((m_FilenameMasks.size() == 1) && (m_FilenameMasks[0] == _T("*.*")))
    {
      if (IsDrive(sPath))
        bAddItem = CanDisplayDrive(sPath);
      else if (bIsFolder)
        bAddItem = ((dwAttributes & m_dwFolderHideFlags) == 0);
      else if (bIsFile)
        bAddItem = ((dwAttributes & m_dwFileHideFlags) == 0);
      else
       bAddItem = false;
    }
    else
    {
      if (IsDrive(sPath))
        bAddItem = CanDisplayDrive(sPath);
      else if (bIsFolder)
        bAddItem = CanDisplayFolder(sPath);
      else if (bIsFile)
        bAddItem = CanDisplayFile(sPath);
      else
        bAddItem = false;
    }
  }
  else
    bAddItem = true;

  return bAddItem;
}

#pragma warning(suppress: 26429)
void CShellTreeCtrl2::OnDblClk(NMHDR* /*pNMHDR*/, LRESULT* pResult)
{
  //Validate our parameters
  ASSERT(pResult != nullptr);

#pragma warning(suppress: 26462)
  HTREEITEM hItem{GetSelectedItem()};
  CPoint pt{GetCurrentMessage()->pt};
  ScreenToClient(&pt);
  if (hItem && (hItem == HitTest(pt)) && m_bAllowDoShellDefault)
  {
    DoShellDefault(GetSelectedItem());
    *pResult = 1;
  }
  else
    *pResult = 0;
}

void CShellTreeCtrl2::Refresh()
{
  if (GetSafeHwnd() != nullptr)
  {
    //Just in case this will take some time
    CWaitCursor wait;

    SetRedraw(FALSE);

    //Get the item which is currently selected
    HTREEITEM hOldSelectedItem{GetSelectedItem()};
    CPIDL oldSelectedPIDL;
    bool bOldSelectedExpanded{false};
    if (hOldSelectedItem)
    {
      TVITEM tvItem{};
      tvItem.hItem = hOldSelectedItem;
      tvItem.mask = TVIF_HANDLE | TVIF_PARAM;
      if (!GetItem(&tvItem))
      {
        ASSERT(FALSE);
      }
      else
      {
#pragma warning(suppress: 26490)
        const CShellItemInfo2* pItem{reinterpret_cast<CShellItemInfo2*>(tvItem.lParam)};
        if (pItem != nullptr)
        {
  #pragma warning(suppress: 26489)
          oldSelectedPIDL = pItem->m_pidlFQ;
          bOldSelectedExpanded = IsExpanded(hOldSelectedItem);
        }
        else
        {
          ASSERT(FALSE);
        }
      }
    }

    KillMonitoringThread();

    //Clear and reshow the items
    DeleteAllItems();
    m_NextItems.clear();
    m_PrevItems.clear();
    GetRootItems();

    //Reselect the initially selected item
    if (hOldSelectedItem)
      SelectPath(oldSelectedPIDL, bOldSelectedExpanded);

    SetRedraw(TRUE);
    RedrawWindow();
  }
}

bool CShellTreeCtrl2::GetRootItems()
{
  if (m_bShowRootedFolder)
  {
  #pragma warning(suppress: 26400 26403 26409)
    auto pItem{new CShellItemInfo2()};
    if (m_RootFolder.operator bool())
    {
      //Put the private information in the item data
      pItem->m_pidlFQ = m_RootFolder;
      CPIDL parent;
      m_RootFolder.Split(parent, pItem->m_pidlRel);
      if (FAILED(SHBindToParent(pItem->m_pidlFQ, IID_IShellFolder, reinterpret_cast<void**>(&pItem->m_pParentFolder), nullptr)))
        return false;
    }
    else
    {
      if (FAILED(SHGetKnownFolderIDList(FOLDERID_Desktop, 0, nullptr, pItem->m_pidlFQ)))
        return false;

      //Put the private information in the item data
      pItem->m_pidlRel = pItem->m_pidlFQ;
      pItem->m_pParentFolder = nullptr; //The IShellFolder for the Desktop will be stored in the item data as null. There is code in CShellTreeCtrl2::GetChildItems to handle this special case.
    }

    //Fill in the TVITEM structure for this item:
    TVITEM tvItem{};
    tvItem.mask = TVIF_PARAM | TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_CHILDREN;
  #pragma warning(suppress: 26490)
    tvItem.lParam = reinterpret_cast<LPARAM>(pItem);
    CString sItem{OnGetItemText(pItem)};
    tvItem.pszText = sItem.GetBuffer(sItem.GetLength());
    tvItem.iImage = OnGetItemIcon(pItem, false);
    tvItem.iSelectedImage = OnGetItemIcon(pItem, true);
    tvItem.cChildren = TRUE; //Assume the desktop has children

    //Fill in the TVINSERTSTRUCT structure for this item
    TVINSERTSTRUCT tvInsert{};
    tvInsert.item = tvItem;
    tvInsert.hInsertAfter = TVI_LAST;
    tvInsert.hParent = TVI_ROOT;

    //Finally add the item
    HTREEITEM hParentItem{InsertItem(&tvInsert)};
    sItem.ReleaseBuffer();
    if (hParentItem == nullptr)
      return false;

    //Go ahead and expand this item
    Expand(hParentItem, TVE_EXPAND);
  }
  else
  {
    if (!GetChildItems(TVI_ROOT))
      return false;
  }

  return true;
}

bool CShellTreeCtrl2::IsExpanded(_In_ HTREEITEM hItem)
{
  TVITEM tvItem{};
  tvItem.hItem = hItem;
  tvItem.mask = TVIF_HANDLE | TVIF_STATE;
  return GetItem(&tvItem) && (tvItem.state & TVIS_EXPANDED);
}

#pragma warning(suppress: 26429)
void CShellTreeCtrl2::OnCustomDraw(NMHDR* pNMHDR, LRESULT* pResult)
{
  //Validate our parameters
  ASSERT(pResult != nullptr);

#pragma warning(suppress: 26490)
  auto pCustomDraw{reinterpret_cast<NMTVCUSTOMDRAW*>(pNMHDR)};
  ASSERT(pCustomDraw != nullptr);
  switch (pCustomDraw->nmcd.dwDrawStage)
  {
    case CDDS_PREPAINT:
    {
      *pResult = CDRF_NOTIFYITEMDRAW; //Tell the control that we are interested in item notifications
      break;
    }
    case CDDS_ITEMPREPAINT:
    {
      //Let it know that we are finished doing custom draw
      *pResult = CDRF_DODEFAULT;

      //To work around what looks like a bug in Commctrl v6 we do not do custom drawing until the window is visible
      if (!IsWindowVisible())
        return;

#pragma warning(suppress: 26490)
      if (m_bShowCompressedUsingDifferentColor && ((pCustomDraw->nmcd.uItemState & CDIS_SELECTED) == 0) && IsCompressed(reinterpret_cast<HTREEITEM>(pCustomDraw->nmcd.dwItemSpec)))
        pCustomDraw->clrText = m_rgbCompressed;
#pragma warning(suppress: 26490)
      else if (m_bShowEncryptedUsingDifferentColor && ((pCustomDraw->nmcd.uItemState & CDIS_SELECTED) == 0) && IsEncrypted(reinterpret_cast<HTREEITEM>(pCustomDraw->nmcd.dwItemSpec)))
        pCustomDraw->clrText = m_rgbEncrypted;

      break;
    }
    default:
    {
      break;
    }
  }
}

#pragma warning(suppress: 26429 26461)
void CShellTreeCtrl2::OnSelChanged(NMHDR* pNMHDR, LRESULT* pResult)
{
#pragma warning(suppress: 26490)
  auto pNMTreeView{reinterpret_cast<const NMTREEVIEW*>(pNMHDR)};
#pragma warning(suppress: 26496)
  ASSERT(pNMTreeView != nullptr);
  ASSERT(pResult != nullptr);

  *pResult = 0;

  //Add to the prev array the item we were just at
#pragma warning(suppress: 26489)
  if ((pNMTreeView->itemNew.hItem != nullptr) && (pNMTreeView->itemOld.hItem != nullptr) && !m_bUpdatingHistorySelection)
  {
    if (m_PrevItems.size() > m_nMaxHistory)
      m_PrevItems.erase(m_PrevItems.begin());
#pragma warning(suppress: 26486 26489)
    m_PrevItems.push_back(pNMTreeView->itemOld.hItem);
  }

  //Get the item data for the newly selected item
  CShellItemInfo2* pItem{nullptr};
  if (pNMTreeView->itemNew.hItem != nullptr)
  {
    TVITEM tvItem{};
    tvItem.hItem = pNMTreeView->itemNew.hItem;
    tvItem.mask = TVIF_HANDLE | TVIF_PARAM;
    if (!GetItem(&tvItem))
    {
      ASSERT(FALSE);
      return;
    }
  #pragma warning(suppress: 26490)
    pItem = reinterpret_cast<CShellItemInfo2*>(tvItem.lParam);
    if (pItem == nullptr)
    {
      ASSERT(FALSE);
      return;
    }
  }

  //Setup notifications if required
  CString sPath;
  if (m_bAutoRefresh && (pItem != nullptr) && pItem->m_pidlFQ.ToPath(sPath) && CanHandleChangeNotifications(sPath))
#pragma warning(suppress: 26489)
    CreateMonitoringThread(sPath);
  else
    KillMonitoringThread();

  //call the virtual function for the selection change
#pragma warning(suppress: 26486 26489)
  OnSelectionChanged(pNMTreeView);

  //Update the display of the related shell list control if necessary
  CShellListCtrl2* pRelatedShellList{GetRelatedList()};
  if (!m_bNoNotify && (pRelatedShellList != nullptr) && (pItem != nullptr))
  {
    pRelatedShellList->m_bNoNotify = true;
#pragma warning(suppress: 26486)
    pRelatedShellList->DisplayFolder(pItem);
    pRelatedShellList->m_bNoNotify = false;
  }
}

#pragma warning(suppress: 26440)
void CShellTreeCtrl2::OnSelectionChanged(_In_ const NMTREEVIEW* /*pNMTreeView*/)
{
  //CShellTreeCtrl2 does nothing, but a derived class may want to do something
}

BOOL CShellTreeCtrl2::PreTranslateMessage(MSG* pMsg)
{
  //When an item is being edited make sure the edit control
  //receives certain important key strokes
  if (GetEditControl())
  {
    ::TranslateMessage(pMsg);
    ::DispatchMessage(pMsg);
    return TRUE; //DO NOT process further
  }

  //Context menu via the keyboard
  if (((((pMsg->message == WM_KEYDOWN) || (pMsg->message == WM_SYSKEYDOWN)) &&
      (pMsg->wParam == VK_F10) && (GetKeyState(VK_SHIFT) & 0x8000)) != 0) ||
      (pMsg->message == WM_CONTEXTMENU))
  {
    CRect rect;
    GetItemRect(GetSelectedItem(), rect, TRUE);
    ClientToScreen(rect);
    OnContextMenu(nullptr, rect.CenterPoint());
    return TRUE;
  }
  //Hitting the Enter key opens the item
  else if ((pMsg->message == WM_KEYDOWN) && (pMsg->wParam == VK_RETURN))
  {
    if (m_bAllowDoShellDefault)
    {
      DoShellDefault(GetSelectedItem());
      return TRUE;
    }
  }
  //hitting the F5 key forces a refresh of the whole tree
  else if ((pMsg->message == WM_KEYDOWN) && (pMsg->wParam == VK_F5))
  {
    Refresh();
    return TRUE;
  }
  //Hitting the Alt-Left Arrow key combination moves to the previous item
  else if ((pMsg->message == WM_SYSKEYDOWN) && (pMsg->wParam == VK_LEFT))
  {
    GoBack();
    return TRUE;
  }
  //Hitting the Alt-Right Arrow key combination moves to the next item
  else if ((pMsg->message == WM_SYSKEYDOWN) && (pMsg->wParam == VK_RIGHT))
  {
    GoForward();
    return TRUE;
  }
  //hitting the backspace key goes to the parent folder
  else if ((pMsg->message == WM_KEYDOWN) && (pMsg->wParam == VK_BACK))
  {
    UpOneLevel();
    return TRUE;
  }

  //Let the parent class do its thing
  return __super::PreTranslateMessage(pMsg);
}

void CShellTreeCtrl2::UpOneLevel()
{
  HTREEITEM hItem{GetSelectedItem()};
  if (hItem)
  {
    HTREEITEM hParent{GetParentItem(hItem)};
    if (hParent != nullptr)
      Select(hParent, TVGN_CARET);
  }
}

bool CShellTreeCtrl2::IsCompressed(_In_ HTREEITEM hItem)
{
  TVITEM tvItem{};
  tvItem.hItem = hItem;
  tvItem.mask = TVIF_HANDLE | TVIF_PARAM;
  if (!GetItem(&tvItem))
  {
    ASSERT(FALSE);
    return false;
  }
#pragma warning(suppress: 26490)
  const CShellItemInfo2* pInfo{reinterpret_cast<CShellItemInfo2*>(tvItem.lParam)};
  if (pInfo == nullptr)
  {
    ASSERT(FALSE);
    return false;
  }
  ASSERT(pInfo != nullptr);
#pragma warning(suppress: 26489)
  return (pInfo->m_Attribs & SFGAO_COMPRESSED);
}

bool CShellTreeCtrl2::IsEncrypted(_In_ HTREEITEM hItem)
{
  TVITEM tvItem{};
  tvItem.hItem = hItem;
  tvItem.mask = TVIF_HANDLE | TVIF_PARAM;
  if (!GetItem(&tvItem))
  {
    ASSERT(FALSE);
    return false;
  }
#pragma warning(suppress: 26490)
  const CShellItemInfo2* pInfo{reinterpret_cast<CShellItemInfo2*>(tvItem.lParam)};
  if (pInfo == nullptr)
  {
    ASSERT(FALSE);
    return false;
  }
#pragma warning(suppress: 26489)
  return (pInfo->m_Attribs & SFGAO_ENCRYPTED);
}

#pragma warning(suppress: 26429)
bool CShellTreeCtrl2::IsDrive(_In_z_ LPCTSTR pszPath) noexcept
{
#pragma warning(suppress: 26481)
  return ((_tcslen(pszPath) == 3) && (pszPath[1] == _T(':')) && (pszPath[2] == _T('\\')));
}

bool CShellTreeCtrl2::GoBack()
{
  bool bSuccess{false};
  const auto nSize{m_PrevItems.size()};
  if (nSize)
  {
    HTREEITEM hOldItem{GetSelectedItem()};
#pragma warning(suppress: 26446)
    HTREEITEM hNewItem{m_PrevItems[nSize - 1]};

    //Select the previous item
    m_bUpdatingHistorySelection = true;
    m_PrevItems.erase(m_PrevItems.begin() + nSize - 1);
    ASSERT(hNewItem != TVI_ROOT);
#pragma warning(suppress: 26486)
    SelectItem(hNewItem);
    EnsureVisible(hNewItem);
    m_bUpdatingHistorySelection = false;

    //Add the old item to the next stack
    m_NextItems.push_back(hOldItem);

    bSuccess = true;
  }

  return bSuccess;
}

bool CShellTreeCtrl2::GoForward()
{
  bool bSuccess{false};
  const auto nSize{m_NextItems.size()};
  if (nSize)
  {
    HTREEITEM hOldItem{GetSelectedItem()};
#pragma warning(suppress: 26446)
    HTREEITEM hNewItem{m_NextItems[nSize - 1]};

    //Select the previous item
    m_bUpdatingHistorySelection = true;
    m_NextItems.erase(m_NextItems.begin() + nSize - 1);
    ASSERT(hNewItem != TVI_ROOT);
#pragma warning(suppress: 26486)
    SelectItem(hNewItem);
    EnsureVisible(hNewItem);
    m_bUpdatingHistorySelection = false;

    //Add the old item to the prev stack
    m_PrevItems.push_back(hOldItem);

    bSuccess = true;
  }

  return bSuccess;
}

void CShellTreeCtrl2::SetMaxHistory(_In_ size_t nMaxHistory) noexcept
{
  m_nMaxHistory = nMaxHistory;

  //Shrink the prev array if necessary
  auto nCurItems{m_PrevItems.size()};
  if (nCurItems > m_nMaxHistory)
    m_PrevItems.erase(m_PrevItems.begin() + m_nMaxHistory, m_PrevItems.end());

  //Shrink the next array if necessary
  nCurItems = m_NextItems.size();
  if (nCurItems > m_nMaxHistory)
    m_NextItems.erase(m_NextItems.begin() + m_nMaxHistory, m_NextItems.end());
}

size_t CShellTreeCtrl2::GetBackSize() const noexcept
{
  return m_PrevItems.size();
}

size_t CShellTreeCtrl2::GetForwardSize() const noexcept
{
  return m_NextItems.size();
}

void CShellTreeCtrl2::CollapseExpandBranch(_In_ HTREEITEM hItem, _In_ int nAction)
{
  if (ItemHasChildren(hItem))
  {
    Expand(hItem, nAction);
    hItem = GetChildItem(hItem);
    while (hItem != nullptr)
    {
      CollapseExpandBranch(hItem, nAction);
      hItem = GetNextSiblingItem(hItem);
    }
  }
}

void CShellTreeCtrl2::CollapseAll()
{
  CWaitCursor wait;

  SetRedraw(FALSE);
  HTREEITEM hItem{GetRootItem()};
  while (hItem != nullptr) //NOLINT(clang-analyzer-core.CallAndMessage)
  {
    CollapseExpandBranch(hItem, TVE_COLLAPSE);
    hItem = GetNextSiblingItem(hItem);
  }
  SetRedraw(TRUE);
  RedrawWindow();
}

void CShellTreeCtrl2::ExpandAll()
{
  CWaitCursor wait;

  SetRedraw(FALSE);
  HTREEITEM hItem{GetRootItem()};
  while (hItem != nullptr) //NOLINT(clang-analyzer-core.CallAndMessage)
  {
    CollapseExpandBranch(hItem, TVE_EXPAND);
    hItem = GetNextSiblingItem(hItem);
  }
  SetRedraw(TRUE);
  RedrawWindow();
}

#pragma warning(suppress: 26434)
int CShellTreeCtrl2::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
  if (__super::OnCreate(lpCreateStruct) == -1) //Let the base class do its thing
    return -1;
  if (!InitTree())
    return -1;
  return 0;
}

bool CShellTreeCtrl2::GetChildItems(_In_ HTREEITEM hParentItem)
{
  CWaitCursor wait;

  if (hParentItem == TVI_ROOT)
  {
    SetRedraw(FALSE);

    ATL::CComPtr<IShellFolder> pParentFolder;
    CPIDL parentPIDL;
    if (m_RootFolder.operator bool())
    {
      parentPIDL = m_RootFolder;
      if (FAILED(SHBindToObject(nullptr, m_RootFolder, nullptr, IID_IShellFolder, reinterpret_cast<void**>(&pParentFolder))))
      {
        SetRedraw(TRUE);
        return false;
      }
    }
    else
    {
      if (FAILED(SHGetDesktopFolder(&pParentFolder)))
      {
        SetRedraw(TRUE);
        return false;
      }
      if (FAILED(SHGetKnownFolderIDList(FOLDERID_Desktop, 0, nullptr, parentPIDL)))
      {
        SetRedraw(TRUE);
        return false;
      }
    }

#pragma warning(suppress: 6387)
    EnumObjects(hParentItem, pParentFolder, parentPIDL);

    //Sort the new items
    TVSORTCB tvSort;
    tvSort.hParent = hParentItem;
    tvSort.lpfnCompare = CompareProc;
    tvSort.lParam = 0;
    SortChildrenCB(&tvSort);

    SetRedraw(TRUE);
    RedrawWindow();
  }
  else
  {
    //Get the parent item's pidl
    TVITEM tvItem{};
    tvItem.hItem = hParentItem;
    tvItem.mask = TVIF_HANDLE | TVIF_PARAM;
    if (!GetItem(&tvItem))
    {
      ASSERT(FALSE);
      return false;
    }
  #pragma warning(suppress: 26429 26490)
    auto pItem{reinterpret_cast<CShellItemInfo2*>(tvItem.lParam)};
    if (pItem == nullptr)
    {
      ASSERT(FALSE);
      return false;
    }

    SetRedraw(FALSE);

    //If the parent folder is null, then we are at the root
    //of the namespace, so the parent of this item is the desktop folder
    ATL::CComPtr<IShellFolder> pParentFolder;
    HRESULT hr{S_OK};
  #pragma warning(suppress: 26489)
    if (pItem->m_pParentFolder == nullptr)
      hr = SHGetDesktopFolder(&pParentFolder);
    else
    {
      //Otherwise we need to get the IShellFolder for this item
  #pragma warning(suppress: 26489 26490)
      hr = pItem->m_pParentFolder->BindToObject(pItem->m_pidlRel, nullptr, IID_IShellFolder, reinterpret_cast<void**>(&pParentFolder));
    }

    if (FAILED(hr))
    {
      SetRedraw(TRUE);
      return FALSE;
    }

  #pragma warning(suppress: 26489)
    EnumObjects(hParentItem, pParentFolder, pItem->m_pidlFQ);

    //Sort the new items
    TVSORTCB tvSort;
    tvSort.hParent = hParentItem;
    tvSort.lpfnCompare = CompareProc;
    tvSort.lParam = 0;
    SortChildrenCB(&tvSort);

    SetRedraw(TRUE);
    RedrawWindow();
  }

  return true;
}

#pragma warning(suppress: 26440)
void CShellTreeCtrl2::OnItemAdded(_In_ HTREEITEM /*hItem*/)
{
  //Derived classes are free to do work here
}

#pragma warning(suppress: 26440)
bool CShellTreeCtrl2::HasChildren(_Inout_ CShellItemInfo2& /*item*/, _In_ SFGAOF attribs)
{
  return (attribs & (SFGAO_HASSUBFOLDER | SFGAO_FILESYSANCESTOR));
}

void CShellTreeCtrl2::PrepareTreeItem(_In_ SFGAOF attribs, _Inout_ CString& sItem, _Inout_ CShellItemInfo2& item, _Inout_ TVITEM& tvItem)
{
  sItem = OnGetItemText(&item);
  tvItem.pszText = sItem.GetBuffer();
  tvItem.iImage = OnGetItemIcon(&item, false);
  tvItem.iSelectedImage = OnGetItemIcon(&item, true);
  tvItem.cChildren = HasChildren(item, attribs);
  //Should we show shared items with a different icon?
  if (m_bShowSharedUsingDifferentIcon && (attribs & SFGAO_SHARE))
  {
    tvItem.mask |= TVIF_STATE;
    tvItem.stateMask |= TVIS_OVERLAYMASK;
    tvItem.state |= INDEXTOOVERLAYMASK(1); //1 is the index for the shared overlay image
  }
  if (m_bShowLinkedItemsUsingDifferentIcon && (attribs & SFGAO_LINK))
  {
    tvItem.mask |= TVIF_STATE;
    tvItem.stateMask |= TVIS_OVERLAYMASK;
    tvItem.state |= INDEXTOOVERLAYMASK(2); //2 is the index for the linked / shortcuts overlay image
  }
  if (attribs & SFGAO_GHOSTED)
  {
    tvItem.mask |= TVIF_STATE;
    tvItem.stateMask |= TVIS_CUT;
    tvItem.state |= TVIS_CUT;
  }
}

HTREEITEM CShellTreeCtrl2::AddItem(_In_ HTREEITEM hParentItem, _In_ ATL::CComPtr<IShellFolder>& pParentFolder, _In_ const CPIDL& pidl, _In_ SFGAOF attribs, _In_ const CPIDL& pidlParent)
{
  //Fill in the TVITEM structure for this item
  TVITEM tvItem{};
  tvItem.mask = TVIF_PARAM | TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_CHILDREN;

  //Put the private information in the lParam
#pragma warning(suppress: 26400 26403 26409)
  auto pItem{new CShellItemInfo2};
  pItem->m_pidlRel = pidl;
  pItem->m_pidlFQ = pidlParent + pidl;
  pItem->m_Attribs = attribs;
  pItem->m_pParentFolder = pParentFolder;
#pragma warning(suppress: 26490)
  tvItem.lParam = reinterpret_cast<LPARAM>(pItem);

  //Fill in the TVINSERTSTRUCT structure for this item
  CString sItem;
  PrepareTreeItem(attribs, sItem, *pItem, tvItem);
  TVINSERTSTRUCT tvInsert{};
  tvInsert.item = tvItem;
  tvInsert.hInsertAfter = TVI_LAST;
  tvInsert.hParent = hParentItem;

  //Finally add the item
  HTREEITEM hNewItem{InsertItem(&tvInsert)};
  sItem.ReleaseBuffer();

  //Call the virtual OnItemAdded method to allow derived classes to customize our behaviour
  OnItemAdded(hNewItem);

#pragma warning(suppress: 26487)
  return hNewItem;
}

#pragma warning(suppress: 26440)
SFGAOF CShellTreeCtrl2::GetAttributesOfFlags()
{
  //Form the flags we will pass to IShellFolder::GetAttributesOf
  SFGAOF attribs{SFGAO_HASSUBFOLDER | SFGAO_FILESYSANCESTOR | SFGAO_GHOSTED};
  if (m_bShowSharedUsingDifferentIcon)
    attribs |= SFGAO_SHARE;
  if (m_bShowCompressedUsingDifferentColor)
    attribs |= SFGAO_COMPRESSED;
  if (m_bShowEncryptedUsingDifferentColor)
    attribs |= SFGAO_ENCRYPTED;
  if (m_bShowLinkedItemsUsingDifferentIcon)
    attribs |= SFGAO_LINK;
  return attribs;
}

#pragma warning(suppress: 26429)
HRESULT CShellTreeCtrl2::EnumObjects(_In_ HTREEITEM hParentItem, _In_ ATL::CComPtr<IShellFolder>& pParentFolder, _In_ const CPIDL& pidlParent)
{
  //Use IShellFolder and IEnumIDList to enumerate the items in the parent folder
  ATL::CComPtr<IEnumIDList> pEnum;
  const HRESULT hr{pParentFolder->EnumObjects(nullptr, m_EnumObjectFlags, &pEnum)};
  if (FAILED(hr) || (pEnum == nullptr))
    return hr;
  CPIDL pidl;
  DWORD dwFetched{1};
  while (SUCCEEDED(pEnum->Next(1, pidl, &dwFetched)) && dwFetched)
  {
    //Add the item if necessary
    SFGAOF attribs{GetAttributesOfFlags()};
    if (SUCCEEDED(pParentFolder->GetAttributesOf(1, pidl, &attribs)) && CanDisplay(pidlParent, pidl))
      AddItem(hParentItem, pParentFolder, pidl, attribs, pidlParent);

    //Prepare for the next time around
    dwFetched = 0;
    pidl.Free();
  }

  return S_OK;
}

int CALLBACK CShellTreeCtrl2::CompareProc(_In_ LPARAM lParam1, _In_ LPARAM lParam2, _In_ LPARAM /*lParamSort*/)
{
#pragma warning(suppress: 26429 26490)
  const CShellItemInfo2* pItem1{reinterpret_cast<CShellItemInfo2*>(lParam1)};
#pragma warning(suppress: 26429 26490)
  const CShellItemInfo2* pItem2{reinterpret_cast<CShellItemInfo2*>(lParam2)};
#pragma warning(suppress: 26489)
  const HRESULT hr{pItem1->m_pParentFolder->CompareIDs(0, pItem1->m_pidlRel, pItem2->m_pidlRel)};
  if (FAILED(hr))
    return 0;
#pragma warning(suppress: 26472)
  return static_cast<short>(SCODE_CODE(GetScode(hr)));
}

void CShellTreeCtrl2::OnShowContextMenu(_In_ CPoint point)
{
  if (m_ContextMenu2 != nullptr)
    return;
  if (!m_bContextMenu)
    return;
  HTREEITEM hItem{nullptr};
  if ((point.x == -1) && (point.y == -1))
  {
    CRect rectItem;
    if (((hItem = GetSelectedItem()) != nullptr) && GetItemRect(hItem, rectItem, FALSE))
    {
      point.x = rectItem.left;
      point.y = rectItem.bottom + 1;
      ClientToScreen(&point);
    }
  }
  else
  {
    CPoint ptClient{point};
    ScreenToClient(&ptClient);
    UINT nFlags{0};
    hItem = HitTest(ptClient, &nFlags);
  }

  if (hItem == nullptr)
    return;

  TVITEM tvItem{};
  tvItem.hItem = hItem;
  tvItem.mask = TVIF_HANDLE | TVIF_PARAM;
  if (!GetItem(&tvItem))
  {
    ASSERT(FALSE);
    return;
  }
#pragma warning(suppress: 26490)
  const CShellItemInfo2* pInfo{reinterpret_cast<CShellItemInfo2*>(tvItem.lParam)};
  if (pInfo == nullptr)
  {
    ASSERT(FALSE);
    return;
  }

#pragma warning(suppress: 26489)
  ATL::CComPtr<IShellFolder> psfFolder{pInfo->m_pParentFolder};
  if (psfFolder == nullptr)
  {
#pragma warning(suppress: 26496)
    if (FAILED(SHGetDesktopFolder(&psfFolder)))
      return;
  }
  if (psfFolder == nullptr)
    return;
  const CWnd* pParent{GetParent()};
  if (pParent == nullptr)
  {
    ASSERT(FALSE);
    return;
  }
  HWND hwndParent{pParent->GetSafeHwnd()};
  ATL::CComPtr<IContextMenu> pcm;
#pragma warning(suppress: 26490)
  if (FAILED(psfFolder->GetUIObjectOf(hwndParent, 1, pInfo->m_pidlRel, IID_IContextMenu, nullptr, reinterpret_cast<void**>(&pcm))))
    return;

  CMenu popup;
  if (!popup.CreatePopupMenu())
    return;
  if (FAILED(pcm->QueryContextMenu(popup.m_hMenu, 0, 1, 0x7FFF, CMF_NORMAL | CMF_EXPLORE)))
    return;
#pragma warning(suppress: 26490)
  if (FAILED(pcm->QueryInterface(IID_IContextMenu2, reinterpret_cast<void**>(&m_ContextMenu2))))
    return;
#pragma warning(suppress: 26472)
  const UINT nIDCmd{static_cast<UINT>(popup.TrackPopupMenu(TPM_LEFTALIGN | TPM_RETURNCMD | TPM_RIGHTBUTTON, point.x, point.y, this))};
  if (m_ContextMenu2 != nullptr)
    m_ContextMenu2.Release();
  if (nIDCmd == 0)
    return;

  CWaitCursor wait;
  CMINVOKECOMMANDINFO cmi{};
  cmi.cbSize = sizeof(CMINVOKECOMMANDINFO);
  cmi.hwnd = hwndParent;
#pragma warning(suppress: 26472 26490)
  cmi.lpVerb = reinterpret_cast<LPCSTR>(static_cast<INT_PTR>(nIDCmd) - 1);
  cmi.nShow = SW_SHOWNORMAL;
  const HRESULT hr{pcm->InvokeCommand(&cmi)};
  if (SUCCEEDED(hr))
#pragma warning(suppress: 26472)
    pParent->SendMessage(WM_SHELLTREECTRL2_AFTER_SHELL_COMMAND, static_cast<WPARAM>(nIDCmd));
  SetFocus();
  if (m_bRefreshAfterShellCommand)
    Refresh();
}

CString CShellTreeCtrl2::OnGetItemText(_In_ const CShellItemInfo2* pItem)
{
  //Validate our parameters
  ASSERT(pItem != nullptr);

  SHFILEINFO sfi{};
#pragma warning(suppress: 26490)
  if (SHGetFileInfo(reinterpret_cast<LPCTSTR>(pItem->m_pidlFQ.operator LPCITEMIDLIST()), 0, &sfi, sizeof(sfi), SHGFI_PIDL | SHGFI_DISPLAYNAME))
#pragma warning(suppress: 26485)
    return sfi.szDisplayName;
  else
    return _T("???");
}

#pragma warning(suppress: 26440)
int CShellTreeCtrl2::OnGetItemIcon(_In_ const CShellItemInfo2* pItem, _In_ bool bSelected)
{
  if (pItem == nullptr)
  {
    ASSERT(FALSE);
    return -1;
  }

  UINT uiFlags{SHGFI_PIDL | SHGFI_SYSICONINDEX | SHGFI_SMALLICON};
  if (bSelected)
    uiFlags |= SHGFI_OPENICON;
  else
    uiFlags |= SHGFI_LINKOVERLAY;
  SHFILEINFO sfi{};
#pragma warning(suppress: 26490)
  if (SHGetFileInfo(reinterpret_cast<LPCTSTR>(pItem->m_pidlFQ.operator LPCITEMIDLIST()), 0, &sfi, sizeof(sfi), uiFlags))
    return sfi.iIcon;
  return -1;
}

#pragma warning(suppress: 26429)
void CShellTreeCtrl2::OnItemExpanding(NMHDR* pNMHDR, LRESULT* pResult)
{
#pragma warning(suppress: 26490)
  auto pNMTreeView{reinterpret_cast<NM_TREEVIEW*>(pNMHDR)};
  ASSERT(pNMTreeView != nullptr);
  ASSERT(pResult != nullptr);
  HTREEITEM hItem{pNMTreeView->itemNew.hItem};
  ASSERT(hItem != nullptr);
  switch (pNMTreeView->action)
  {
    case TVE_EXPAND:
    {
      if (m_bKeepChildrenOnCollapse)
      {
        if (GetChildItem(hItem) == nullptr)
        {
          GetChildItems(hItem);
          if (GetChildItem(hItem) == nullptr)
          {
            //Remove '+':
            TVITEM tvItem{};
            tvItem.hItem = hItem;
            tvItem.mask = TVIF_CHILDREN;
            SetItem(&tvItem);
          }
        }
      }
      else
      {
        GetChildItems(hItem);
        if (GetChildItem(hItem) == nullptr)
        {
          //Remove '+':
          TVITEM tvItem{};
          tvItem.hItem = hItem;
          tvItem.mask = TVIF_CHILDREN;
          SetItem(&tvItem);
        }
      }
      break;
    }
    case TVE_COLLAPSE:
    {
      for (HTREEITEM hItemSel{GetSelectedItem()}; hItemSel != nullptr; )
      {
        HTREEITEM hParentItem{GetParentItem(hItemSel)};
        if (hParentItem == hItem)
        {
          SelectItem(hItem);
          break;
        }
        hItemSel = hParentItem;
      }

      if (m_bKeepChildrenOnCollapse)
      {
        //Collapse the branch (but do not remove child items)
        Expand(hItem, TVE_COLLAPSE);
      }
      else
      {
        //remove all of the items from this node
        Expand(hItem, TVE_COLLAPSE | TVE_COLLAPSERESET);
      }
      break;
    }
    default:
    {
      break;
    }
  }
  *pResult = 0;
}

#pragma warning(suppress: 26429 26434 26440)
void CShellTreeCtrl2::OnDeleteItem(NMHDR* pNMHDR, LRESULT* pResult)
{
#pragma warning(suppress: 26490)
  auto pNMTreeView{reinterpret_cast<NM_TREEVIEW*>(pNMHDR)};
#pragma warning(suppress: 26496)
  ASSERT(pNMTreeView != nullptr);
  ASSERT(pResult != nullptr);
#pragma warning(suppress: 26429 26490)
  auto pItem{reinterpret_cast<CShellItemInfo2*>(pNMTreeView->itemOld.lParam)};
  ASSERT(pItem != nullptr);
#pragma warning(suppress: 26401 26409 26486 26489)
  delete pItem;
  *pResult = 0;
}

#pragma warning(suppress: 26434)
void CShellTreeCtrl2::OnContextMenu(CWnd* /*pWnd*/, CPoint point)
{
  if (m_bContextMenu)
    OnShowContextMenu(point);
  else
    Default();
}

#pragma warning(suppress: 26487)
HRESULT CShellTreeCtrl2::SelectPath(_In_z_ LPCTSTR pszPath, _In_ bool fExpandToShowChildren, _Out_opt_ HTREEITEM* phItem)
{
  //Validate our parameters
#pragma warning(suppress: 26496)
  ASSERT(pszPath != nullptr);

  //Default the output parameter
  if (phItem != nullptr)
    *phItem = nullptr;

  CPIDL pidl;
  const HRESULT hr{pidl.FromPath(pszPath)};
  if (FAILED(hr))
    return hr;
  return SelectPath(pidl, fExpandToShowChildren, phItem);
}

HRESULT CShellTreeCtrl2::SelectPath(_In_ const CPIDL& pidl, _In_ bool fExpandToShowChildren, _Out_opt_ HTREEITEM* phItem)
{
  if (!pidl) //An empty PIDL does not make sense here!
    return E_INVALIDARG;

  //Default the output parameter
  if (phItem != nullptr)
    *phItem = nullptr;

  HTREEITEM hItem{nullptr};
  SetRedraw(FALSE);
  if (m_bShowRootedFolder && m_RootFolder.operator bool())
  {
    hItem = GetRootItem();
    if (pidl.GetItemCount() != 0)
    {
      //Build up an array of PIDL items to match against the tree items
      std::vector<CPIDL> items;
      items.push_back(pidl);
      CPIDL currentPIDL{pidl};
      CPIDL parentPIDL;
      UINT nCurrentItems{currentPIDL.GetItemCount()};
      while ((nCurrentItems != 1) && (currentPIDL != m_RootFolder))
      {
        const HRESULT hr{currentPIDL.GetParent(parentPIDL)};
        if (FAILED(hr))
        {
          SetRedraw(TRUE);
          return hr;
        }
        items.insert(items.begin(), parentPIDL);
        currentPIDL = parentPIDL;
        --nCurrentItems;
      }

      //Iterate through all the PIDL array items matching to tree items as we go
      for (const auto& tempPIDL : items)
      {
        if (hItem != nullptr)
        {
          if (GetChildItem(hItem) == nullptr)
            Expand(hItem, TVE_EXPAND);
#pragma warning(suppress: 26490)
          auto pItem{reinterpret_cast<CShellItemInfo2*>(__super::GetItemData(hItem))};
          ASSERT(pItem != nullptr);
#pragma warning(suppress: 26489)
          if (pItem->m_pidlFQ == tempPIDL)
            continue;
          bool bFound{false};
          for (HTREEITEM hChild{GetChildItem(hItem)}; !bFound && (hChild != nullptr); hChild = GetNextSiblingItem(hChild))
          {
#pragma warning(suppress: 26462 26490)
            pItem = reinterpret_cast<CShellItemInfo2*>(__super::GetItemData(hChild));
            ASSERT(pItem != nullptr);
            SHFILEINFO sfi1{};
            SHFILEINFO sfi2{};
#pragma warning(suppress: 26490)
            if (SHGetFileInfo(reinterpret_cast<LPCTSTR>(pItem->m_pidlFQ.m_pPIDL), 0, &sfi1, sizeof(sfi1), SHGFI_PIDL | SHGFI_DISPLAYNAME) &&
#pragma warning(suppress: 26485 26490)
                SHGetFileInfo(reinterpret_cast<LPCTSTR>(tempPIDL.m_pPIDL), 0, &sfi2, sizeof(sfi2), SHGFI_PIDL | SHGFI_DISPLAYNAME) && lstrcmpi(sfi1.szDisplayName, sfi2.szDisplayName) == 0)
            {
              bFound = true;
              hItem = hChild;
            }
          }
          if (!bFound)
            hItem = nullptr;
        }
      }
    }
  }
  else if (m_bShowRootedFolder && !m_RootFolder.operator bool())
  {
    hItem = GetRootItem();
    if (pidl.GetItemCount() != 0)
    {
      //Build up an array of PIDL items to match against the tree items
      std::vector<CPIDL> items;
      items.push_back(pidl);
#pragma warning(suppress: 26486)
      CPIDL currentPIDL{pidl};
      CPIDL parentPIDL;
      UINT nCurrentItems{currentPIDL.GetItemCount()};
      while (nCurrentItems != 1)
      {
        const HRESULT hr{currentPIDL.GetParent(parentPIDL)};
        if (FAILED(hr))
        {
          SetRedraw(TRUE);
          return hr;
        }
        items.insert(items.begin(), parentPIDL);
        currentPIDL = parentPIDL;
        --nCurrentItems;
      }

      //Iterate through all the PIDL array items matching to tree items as we go
      for (const auto& tempPIDL : items)
      {
        if (hItem != nullptr)
        {
          if (GetChildItem(hItem) == nullptr)
            Expand(hItem, TVE_EXPAND);
          bool bFound{false};
          for (HTREEITEM hChild{GetChildItem(hItem)}; !bFound && (hChild != nullptr); hChild = GetNextSiblingItem(hChild))
          {
#pragma warning(suppress: 26462 26490)
            const auto pItem{reinterpret_cast<CShellItemInfo2*>(__super::GetItemData(hChild))};
            ASSERT(pItem != nullptr);
            SHFILEINFO sfi1{};
            SHFILEINFO sfi2{};
#pragma warning(suppress: 26490)
            if (SHGetFileInfo(reinterpret_cast<LPCTSTR>(pItem->m_pidlFQ.m_pPIDL), 0, &sfi1, sizeof(sfi1), SHGFI_PIDL | SHGFI_DISPLAYNAME) &&
#pragma warning(suppress: 26485 26490)
                SHGetFileInfo(reinterpret_cast<LPCTSTR>(tempPIDL.m_pPIDL), 0, &sfi2, sizeof(sfi2), SHGFI_PIDL | SHGFI_DISPLAYNAME) && lstrcmpi(sfi1.szDisplayName, sfi2.szDisplayName) == 0)
            {
              bFound = true;
              hItem = hChild;
            }
          }
          if (!bFound)
            hItem = nullptr;
        }
      }
    }
  }
  else if (!m_bShowRootedFolder && m_RootFolder.operator bool())
  {
    //Build up an array of PIDL items to match against the tree items
    std::vector<CPIDL> items;
    items.push_back(pidl);
    CPIDL currentPIDL{pidl};
    CPIDL parentPIDL;
    UINT nCurrentItems{currentPIDL.GetItemCount()};
    while ((nCurrentItems != 1) && (currentPIDL != m_RootFolder))
    {
      const HRESULT hr{currentPIDL.GetParent(parentPIDL)};
      if (FAILED(hr))
      {
        SetRedraw(TRUE);
        return hr;
      }
      if (parentPIDL != m_RootFolder)
      {
#pragma warning(suppress: 26486)
        items.insert(items.begin(), parentPIDL);
      }
      currentPIDL = parentPIDL;
      --nCurrentItems;
    }

    //Iterate through all the PIDL array items matching to tree items as we go
    hItem = TVI_ROOT;
    const auto nItems{items.size()};
    for (size_t i=0; i<nItems; i++)
    {
#pragma warning(suppress: 26446)
      const CPIDL& tempPIDL{items[i]};
      bool bFound{false};
      if (hItem != nullptr)
      {
        for (HTREEITEM hChild{GetChildItem(hItem)}; !bFound && (hChild != nullptr); hChild = GetNextSiblingItem(hChild))
        {
#pragma warning(suppress: 26462 26490)
          const auto pItem{reinterpret_cast<CShellItemInfo2*>(__super::GetItemData(hChild))};
          ASSERT(pItem != nullptr);
          SHFILEINFO sfi1{};
          SHFILEINFO sfi2{};
#pragma warning(suppress: 26490)
          if (SHGetFileInfo(reinterpret_cast<LPCTSTR>(pItem->m_pidlFQ.m_pPIDL), 0, &sfi1, sizeof(sfi1), SHGFI_PIDL | SHGFI_DISPLAYNAME) &&
#pragma warning(suppress: 26485 26490)
              SHGetFileInfo(reinterpret_cast<LPCTSTR>(tempPIDL.m_pPIDL), 0, &sfi2, sizeof(sfi2), SHGFI_PIDL | SHGFI_DISPLAYNAME) && lstrcmpi(sfi1.szDisplayName, sfi2.szDisplayName) == 0)
          {
            bFound = true;
            hItem = hChild;
            if ((i < (nItems - 1)) && (GetChildItem(hItem) == nullptr))
              Expand(hItem, TVE_EXPAND);
          }
        }
        if (!bFound)
          hItem = nullptr;
      }
    }
  }
  else
  {
    //Build up an array of PIDL items to match against the tree items
    std::vector<CPIDL> items;
    items.push_back(pidl);
#pragma warning(suppress: 26486)
    CPIDL currentPIDL{pidl};
    CPIDL parentPIDL;
    UINT nCurrentItems{currentPIDL.GetItemCount()};
    while (nCurrentItems != 1)
    {
      const HRESULT hr{currentPIDL.GetParent(parentPIDL)};
      if (FAILED(hr))
      {
        SetRedraw(TRUE);
        return hr;
      }
      items.insert(items.begin(), parentPIDL);
      currentPIDL = parentPIDL;
      --nCurrentItems;
    }

    //Iterate through all the array items matching to tree items as we go
    hItem = TVI_ROOT;
    const auto nItems{items.size()};
    for (size_t i{0}; i<nItems; i++)
    {
#pragma warning(suppress: 26446)
      const CPIDL& tempPIDL{items[i]};
      bool bFound{false};
      if (hItem != nullptr)
      {
        for (HTREEITEM hTreeChild{GetChildItem(hItem)}; !bFound && (hTreeChild != nullptr); hTreeChild = GetNextSiblingItem(hTreeChild))
        {
#pragma warning(suppress: 26462 26490)
          const auto pItem{reinterpret_cast<CShellItemInfo2*>(__super::GetItemData(hTreeChild))};
          ASSERT(pItem != nullptr);
          SHFILEINFO sfi1{};
          SHFILEINFO sfi2{};
#pragma warning(suppress: 26490)
          if (SHGetFileInfo(reinterpret_cast<LPCTSTR>(pItem->m_pidlFQ.m_pPIDL), 0, &sfi1, sizeof(sfi1), SHGFI_PIDL | SHGFI_DISPLAYNAME) &&
#pragma warning(suppress: 26485 26490)
              SHGetFileInfo(reinterpret_cast<LPCTSTR>(tempPIDL.m_pPIDL), 0, &sfi2, sizeof(sfi2), SHGFI_PIDL | SHGFI_DISPLAYNAME) && lstrcmpi(sfi1.szDisplayName, sfi2.szDisplayName) == 0)
          {
            bFound = true;
            hItem = hTreeChild;
            if ((i < (nItems - 1)) && (GetChildItem(hItem) == nullptr))
              Expand(hItem, TVE_EXPAND);
          }
        }
        if (!bFound)
          hItem = nullptr;
      }
    }
  }

  //Finally select and expand the final node as required
  HRESULT hr{S_FALSE};
  if (hItem != nullptr)
  {
    m_bNoNotify = true;
    SelectItem(hItem);
    if (fExpandToShowChildren && (GetChildItem(hItem) == nullptr))
      Expand(hItem, TVE_EXPAND);
    EnsureVisible(hItem);
    m_bNoNotify = false;
    hr = S_OK;

    //Update the output parameter
    if (phItem != nullptr)
      *phItem = hItem;
  }

  SetRedraw(TRUE);
  RedrawWindow();
  return hr;
}

bool CShellTreeCtrl2::GetItemPIDL(_In_ HTREEITEM hItem, _Inout_ CPIDL& pidl) const
{
  TVITEM tvItem{};
  tvItem.hItem = hItem;
  tvItem.mask = TVIF_HANDLE | TVIF_PARAM;
  if (!GetItem(&tvItem))
  {
    ASSERT(FALSE);
    return false;
  }
#pragma warning(suppress: 26434 26490)
  const CShellItemInfo2* pItem{reinterpret_cast<CShellItemInfo2*>(tvItem.lParam)};
  if (pItem == nullptr)
  {
    ASSERT(FALSE);
    return false;
  }
#pragma warning(suppress: 26489)
  pidl = pItem->m_pidlFQ;
  return true;
}

bool CShellTreeCtrl2::GetItemPath(_In_ HTREEITEM hTreeItem, _Inout_ CString& sPath) const
{
  CPIDL pidl;
  if (!GetItemPIDL(hTreeItem, pidl))
    return false;
  return pidl.ToPath(sPath);
}

#pragma warning(suppress: 26434)
void CShellTreeCtrl2::OnRButtonDown(UINT /*nFlags*/, CPoint point)
{
  SetFocus();
  UINT nFlags{0};
  SelectItem(HitTest(point, &nFlags));
}

#pragma warning(suppress: 26434)
void CShellTreeCtrl2::OnDestroy()
{
  KillMonitoringThread();
  CShellListCtrl2* pList{GetRelatedList()};
  if (pList != nullptr)
    pList->m_hwndRelatedTree = nullptr;
  __super::OnDestroy();
}

LRESULT CShellTreeCtrl2::WindowProc(UINT message, WPARAM wParam, LPARAM lParam)
{
  switch (message)
  {
    case WM_INITMENUPOPUP: //deliberate fallthrough
    case WM_DRAWITEM: //deliberate fallthrough
    case WM_MEASUREITEM:
    {
      if (m_ContextMenu2 != nullptr)
      {
        m_ContextMenu2->HandleMenuMsg(message, wParam, lParam);
        return 0;
      }
      break;
    }
    default:
    {
      break;
    }
  }
  return __super::WindowProc(message, wParam, lParam);
}

void CShellTreeCtrl2::PreSubclassWindow()
{
  //Let the base class do its thing
  __super::PreSubclassWindow();

  const _AFX_THREAD_STATE* pThreadState{AfxGetThreadState()};
  if (pThreadState->m_pWndInit == nullptr)
    InitTree();
}

bool CShellTreeCtrl2::InitTree()
{
  CString sWinDir;
  const UINT nGWD{GetWindowsDirectory(sWinDir.GetBuffer(_MAX_PATH), _MAX_PATH)};
  sWinDir.ReleaseBuffer();
  if (nGWD == 0)
    return false;

  //Attach to the system image list
  SHFILEINFO sfi{};
#pragma warning(suppress: 26490)
  SetImageList(CImageList::FromHandle(reinterpret_cast<HIMAGELIST>(SHGetFileInfo(sWinDir, 0, &sfi, sizeof(SHFILEINFO), SHGFI_SYSICONINDEX | SHGFI_SMALLICON))), TVSIL_NORMAL);

  //Force a refresh
  Refresh();

  return true;
}

void CShellTreeCtrl2::SetEnumObjectFlags(_In_ SHCONTF flags, _In_ bool bRefresh)
{
  if (flags != m_EnumObjectFlags)
  {
    m_EnumObjectFlags = flags;
    if (m_EnumObjectFlags & SHCONTF_NONFOLDERS)
      m_bShowFiles = true;
    else
      m_bShowFiles = false;
    if (bRefresh && (GetSafeHwnd() != nullptr))
      Refresh();
  }
}

#pragma warning(suppress: 26434)
DWORD_PTR CShellTreeCtrl2::GetItemData(_In_ HTREEITEM hItem) const
{
#pragma warning(suppress: 26490)
  const CShellItemInfo2* pItem{reinterpret_cast<CShellItemInfo2*>(__super::GetItemData(hItem))};
  ASSERT(pItem != nullptr);
#pragma warning(suppress: 26489)
  return pItem->m_dwItemData;
}

void* CShellTreeCtrl2::GetItemDataPtr(_In_ HTREEITEM hItem) const
{
#pragma warning(suppress: 26490)
  const CShellItemInfo2* pItem{reinterpret_cast<CShellItemInfo2*>(__super::GetItemData(hItem))};
  ASSERT(pItem != nullptr);
#pragma warning(suppress: 26489 26490)
  return reinterpret_cast<void*>(pItem->m_dwItemData);
}

#pragma warning(suppress: 26434)
bool CShellTreeCtrl2::SetItemData(_In_ HTREEITEM hItem, _In_ DWORD_PTR dwData)
{
  TVITEM tvItem{};
  tvItem.hItem = hItem;
  tvItem.mask = TVIF_HANDLE | TVIF_PARAM;
  if (!GetItem(&tvItem))
  {
    ASSERT(FALSE);
    return false;
  }
#pragma warning(suppress: 26490)
  auto pItem{reinterpret_cast<CShellItemInfo2*>(tvItem.lParam)};
  if (pItem == nullptr)
  {
    ASSERT(FALSE);
    return false;
  }
#pragma warning(suppress: 26489)
  pItem->m_dwItemData = dwData;
  return true;
}

#pragma warning(suppress: 26434)
bool CShellTreeCtrl2::SetItemDataPtr(_In_ HTREEITEM hItem, _In_opt_ void* pData)
{
  TVITEM tvItem{};
  tvItem.hItem = hItem;
  tvItem.mask = TVIF_HANDLE | TVIF_PARAM;
  if (!GetItem(&tvItem))
  {
    ASSERT(FALSE);
    return false;
  }
#pragma warning(suppress: 26490)
  auto pItem{reinterpret_cast<CShellItemInfo2*>(tvItem.lParam)};
  if (pItem == nullptr)
  {
    ASSERT(FALSE);
    return false;
  }
#pragma warning(suppress: 26489 26490)
  pItem->m_dwItemData = reinterpret_cast<DWORD_PTR>(pData);
  return true;
}

CShellListCtrl2* CShellTreeCtrl2::GetRelatedList() const
{
  if ((m_hwndRelatedList == nullptr) || !::IsWindow(m_hwndRelatedList))
    return nullptr;
  return DYNAMIC_DOWNCAST(CShellListCtrl2, CWnd::FromHandlePermanent(m_hwndRelatedList));
}

void CShellTreeCtrl2::SetRelatedList(_In_opt_ CShellListCtrl2* pShellList)
{
  m_hwndRelatedList = (pShellList == nullptr) ? nullptr : pShellList->GetSafeHwnd();
  if (pShellList != nullptr)
    pShellList->m_hwndRelatedTree = GetSafeHwnd();
}

void CShellTreeCtrl2::CreateMonitoringThread(_In_z_ LPCTSTR pszPath)
{
  //Kill any current notification thread
  KillMonitoringThread();

  //Setup the monitoring path
  m_sMonitoringPath = pszPath;
  ASSERT(m_pMonitoringThread == nullptr);
  m_pMonitoringThread = AfxBeginThread(_MonitoringThread, this, THREAD_PRIORITY_IDLE, 0, CREATE_SUSPENDED);
  if (m_pMonitoringThread != nullptr)
  {
    //We're in charge of deletion of the thread
    m_pMonitoringThread->m_bAutoDelete = FALSE;

    //Resume the thread now that everything is ready to go
    m_pMonitoringThread->ResumeThread();
  }
}

UINT CShellTreeCtrl2::_MonitoringThread(_In_ LPVOID pParam)
{
  //Validate our parameters
  ASSERT(pParam != nullptr);
  auto pThis{static_cast<CShellTreeCtrl2*>(pParam)};
#pragma warning(suppress: 26496)
  AFXASSUME(pThis != nullptr);

  //Convert from the SDK world to the C++ world
  return pThis->MonitoringThread(); //NOLINT(clang-analyzer-core.CallAndMessage)
}

UINT CShellTreeCtrl2::MonitoringThread()
{
  //Form the notification flag to use
  DWORD dwNotifyFilter{FILE_NOTIFY_CHANGE_DIR_NAME};
  if (m_bShowFiles)
    dwNotifyFilter |= FILE_NOTIFY_CHANGE_FILE_NAME;

  //Get a handle to a file change notification object
  HANDLE hChange{FindFirstChangeNotification(m_sMonitoringPath, TRUE, dwNotifyFilter)};
  if (hChange != INVALID_HANDLE_VALUE)
  {
    //Setup the handles to wait on
    std::array<HANDLE, 2> handles{ { hChange, m_MonitoringTerminateEvent } };

    //Sleep until a file change notification wakes this thread or m_TerminateEvent becomes
    //set indicating it's time for the thread to end
    bool bContinue{true};
    while (bContinue)
    {
#pragma warning(suppress: 26472)
      const DWORD dwWait{WaitForMultipleObjects(static_cast<DWORD>(handles.size()), handles.data(), FALSE, INFINITE)};
      if ((dwWait - WAIT_OBJECT_0) == 0)
      {
        //Respond to the change notification by posting a user defined message back to the GUI thread
        if (!m_bAutoRefresh)
          bContinue = false;
        else
          PostMessage(WM_SHELLTREECTRL2_NOTIFICATION_CHANGE);

        //Move onto the next notification
        FindNextChangeNotification(hChange);
      }
      else
      {
        //Kill the thread
        bContinue = false;
      }
    }

    //Close the handle we have open
    FindCloseChangeNotification(hChange);
  }

  return 0;
}

void CShellTreeCtrl2::KillMonitoringThread()
{
  if (m_pMonitoringThread != nullptr)
  {
    //Signal the worker thread to exit and wait for it to return
    m_MonitoringTerminateEvent.SetEvent();
    WaitForSingleObject(m_pMonitoringThread->m_hThread, INFINITE);
    delete m_pMonitoringThread;
    m_pMonitoringThread = nullptr;
  }
}

LRESULT CShellTreeCtrl2::OnNotificationChange(WPARAM /*wParam*/, LPARAM /*lParam*/)
{
  if (m_bAutoRefresh)
    Refresh();
  return 0L;
}

bool CShellTreeCtrl2::CanHandleChangeNotifications(_In_z_ LPCTSTR pszPath)
{
  //The path must be a folder of some kind
  const DWORD dwAttributes{GetFileAttributes(pszPath)};
  if (dwAttributes == INVALID_FILE_ATTRIBUTES)
    return false;
  if ((dwAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
    return false;

  //check if this drive is one of the types which can issue notification changes
  CString sDrive;
  _tsplitpath_s(pszPath, sDrive.GetBuffer(_MAX_DRIVE), _MAX_DRIVE, nullptr, 0, nullptr, 0, nullptr, 0);
  sDrive.ReleaseBuffer();
  const UINT nDrive{GetDriveType(sDrive)};
  return ((nDrive != DRIVE_REMOVABLE) && (nDrive != DRIVE_CDROM));
}

void CShellTreeCtrl2::SetAutoRefresh(_In_ bool bAutoRefresh)
{
  if (bAutoRefresh != m_bAutoRefresh)
  {
    m_bAutoRefresh = bAutoRefresh;

    //Force the monitoring thread to be recreated
    Refresh();
  }
}
