/*
Module : ShellTreeCtrl2.h
Purpose: Interface for a CShellTreeCtrl2 class which is an improved version of the built-in MFC class CMFCShellTreeCtrl

Copyright (c) 2020 - 2023 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code.

*/


//////////////////// Macros / Defines /////////////////////////////////////////

#pragma once

#ifndef __SHELLTREECTRL2_H__
#define __SHELLTREECTRL2_H__

#ifndef SHELLTREECTRL2_EXT_CLASS
#define SHELLTREECTRL2_EXT_CLASS
#endif //#ifndef SHELLTREECTRL2_EXT_CLASS

#ifndef SHELLTREECTRL2_BASE_CLASS
#define SHELLTREECTRL2_BASE_CLASS CTreeCtrl
#endif //#ifndef SHELLTREECTRL2_BASE_CLASS


//////////////////// Includes /////////////////////////////////////////////////

#ifndef _ATOMIC_
#pragma message("To avoid this message, put atomic in your pre compiled header (normally stdafx.h)")
#include <atomic>
#endif //#ifndef _ATOMIC_

#ifndef __AFXMT_H__
#pragma message("To avoid this message, put afxmt.h in your pre compiled header (normally stdafx.h)")
#include <afxmt.h>
#endif //#ifndef __AFXMT_H__

#include "ShellItemInfo2.h"


//////////////////// Classes //////////////////////////////////////////////////

class CShellListCtrl2;

//Class which implements the tree control representation of the file system
class SHELLTREECTRL2_EXT_CLASS CShellTreeCtrl2 : public SHELLTREECTRL2_BASE_CLASS
{
public:
//Enums
  enum DriveType
  {
    DRIVE_ATTRIBUTE_REMOVABLE = 0x00000001,
    DRIVE_ATTRIBUTE_FIXED = 0x00000002,
    DRIVE_ATTRIBUTE_REMOTE = 0x00000004,
    DRIVE_ATTRIBUTE_CDROM = 0x00000010,
    DRIVE_ATTRIBUTE_RAMDISK = 0x00000020
  };

//Constructors / Destructors
  CShellTreeCtrl2();
  CShellTreeCtrl2(const CShellTreeCtrl2&) = delete;
  CShellTreeCtrl2(CShellTreeCtrl2&&) = delete;
  ~CShellTreeCtrl2() = default; //NOLINT(modernize-use-override)

//Methods
  CShellTreeCtrl2& operator=(const CShellTreeCtrl2&) = delete;
  CShellTreeCtrl2& operator=(CShellTreeCtrl2&&) = delete;
  HRESULT SetRootFolder(_In_z_ LPCTSTR pszRoot);
  void SetRootFolder(_In_ const CPIDL& root);
  _NODISCARD CPIDL GetRootFolder() const { return m_RootFolder; };
  HRESULT SelectPath(_In_z_ LPCTSTR pszPath, _In_ bool fExpandToShowChildren = true, _Out_opt_ HTREEITEM* phItem = nullptr);
  HRESULT SelectPath(_In_ const CPIDL& pidl, _In_ bool fExpandToShowChildren = true, _Out_opt_ HTREEITEM* phItem = nullptr);
  bool GetItemPath(_In_ HTREEITEM hItem, _Inout_ CString& sPath) const;
  bool GetItemPIDL(_In_ HTREEITEM hItem, _Inout_ CPIDL& pidl) const;
  void SetShowFiles(_In_ bool bShowFiles, _In_ bool bRefresh = true);
  _NODISCARD bool GetShowFiles() const noexcept { return m_bShowFiles; };
  void SetUsingDifferentIconForSharedFolders(_In_ bool bShowSharedUsingDifferentIcon, _In_ bool bRefresh = true);
  _NODISCARD bool GetUsingDifferentIconForSharedFolders() const noexcept { return m_bShowSharedUsingDifferentIcon; };
  void SetUsingDifferentIconForLinkedItems(_In_ bool bShowLinkedItemsUsingDifferentIcon, _In_ bool bRefresh = true);
  _NODISCARD bool GetUsingDifferentIconForLinkedItems() const noexcept { return m_bShowLinkedItemsUsingDifferentIcon; };
  void SetAllowDoShellDefault(_In_ bool bAllowDoShellDefault) noexcept { m_bAllowDoShellDefault = bAllowDoShellDefault; };
  _NODISCARD bool GetAllowDoShellDefault() const noexcept { return m_bAllowDoShellDefault; };
  void SetDriveHideFlags(_In_ DWORD dwDriveHideFlags, _In_ bool bRefresh = true);
  _NODISCARD DWORD GetDriveHideFlags() const noexcept { return m_dwDriveHideFlags; };
  void SetFileHideFlags(_In_ DWORD dwFileHideFlags, _In_ bool bRefresh = true);
  _NODISCARD DWORD GetFileHideFlags() const noexcept { return m_dwFileHideFlags; };
  void SetFolderHideFlags(_In_ DWORD dwFolderHideFlags, _In_ bool bRefresh = true);
  _NODISCARD DWORD GetFolderHideFlags() const noexcept { return m_dwFolderHideFlags; };
  size_t AddMultipleFilenameMasks(_In_ const CString& sFileNameMasks, _In_ bool bRefresh = true);
  void SetFilenameMask(_In_ size_t nIndex, _In_ const CString& sFilenameMask, _In_ bool bRefresh = true);
  void AddFilenameMask(_In_ const CString& sFilenameMask, _In_ bool bRefresh = true);
  void RemoveFilenameMask(_In_ int nIndex) { m_FilenameMasks.erase(m_FilenameMasks.begin() + nIndex); };
#pragma warning(suppress: 26446)
  _NODISCARD CString GetFilenameMask(_In_ int nIndex) const { return m_FilenameMasks[nIndex]; };
  _NODISCARD CString GetFilenameMasks() const;
  _NODISCARD COLORREF GetCompressedColor() const noexcept { return m_rgbCompressed; };
  void SetCompressedColor(_In_ COLORREF rgbCompressed, _In_ bool bRefresh = true);
  _NODISCARD bool GetUsingDifferentColorForCompressed() const noexcept { return m_bShowCompressedUsingDifferentColor; };
  void SetUsingDifferentColorForCompressed(_In_ bool bShowCompressedUsingDifferentColor, _In_ bool bRefresh = true);
  _NODISCARD COLORREF GetEncryptedColor() const noexcept { return m_rgbEncrypted; };
  void SetEncryptedColor(_In_ COLORREF rgbEncrypted, _In_ bool bRefresh = true);
  _NODISCARD bool GetUsingDifferentColorForEncrypted() const noexcept { return m_bShowEncryptedUsingDifferentColor; };
  void SetUsingDifferentColorForEncrypted(_In_ bool bShowEncryptedUsingDifferentColor, _In_ bool bRefresh = true);
  _NODISCARD bool GetShowRootedFolder() const noexcept { return m_bShowRootedFolder; };
  void SetShowRootedFolder(_In_ bool bShowRootedFolder, _In_ bool bRefresh = true);
  void SetKeepChildrenOnCollapse(_In_ bool bKeepChildrenOnCollapse) noexcept { m_bKeepChildrenOnCollapse = bKeepChildrenOnCollapse; };
  _NODISCARD bool GetKeepChildrenOnCollapse() const noexcept { return m_bKeepChildrenOnCollapse; };
  _NODISCARD virtual bool IsCompressed(_In_ HTREEITEM hItem);
  _NODISCARD virtual bool IsEncrypted(_In_ HTREEITEM hItem);
  _NODISCARD static bool IsDrive(_In_z_ LPCTSTR pszPath) noexcept;
  virtual bool DoShellDefault(_In_ HTREEITEM hItem);
  virtual bool DoShellProperties(_In_ HTREEITEM hItem);
  virtual bool InitTree();
  virtual void UpOneLevel();
  virtual void Refresh();
  virtual bool GoBack();
  _NODISCARD virtual bool CanGoBack() const noexcept { return m_PrevItems.size() != 0; };
  virtual bool GoForward();
  _NODISCARD virtual bool CanGoForward() const noexcept { return m_NextItems.size() != 0; };
  _NODISCARD virtual size_t GetMaxHistory() const noexcept { return m_nMaxHistory; };
  virtual void SetMaxHistory(_In_ size_t nMaxHistory) noexcept;
  _NODISCARD size_t GetBackSize() const noexcept;
  _NODISCARD size_t GetForwardSize() const noexcept;
  void CollapseExpandBranch(_In_ HTREEITEM hItem, _In_ int nAction);
  void CollapseAll();
  void ExpandAll();
  _NODISCARD SHCONTF GetEnumObjectFlags() const noexcept { return m_EnumObjectFlags; }
  void SetEnumObjectFlags(_In_ SHCONTF flags, _In_ bool bRefresh = true);
  void EnableShellContextMenu(_In_ bool bEnable) noexcept { m_bContextMenu = bEnable; };
  _NODISCARD CShellListCtrl2* GetRelatedList() const;
  void SetRelatedList(_In_opt_ CShellListCtrl2* pShellList);
  _NODISCARD DWORD_PTR GetItemData(_In_ HTREEITEM hItem) const;
  _NODISCARD void* GetItemDataPtr(_In_ HTREEITEM hItem) const;
  bool SetItemData(_In_ HTREEITEM hItem, _In_ DWORD_PTR dwData);
  bool SetItemDataPtr(_In_ HTREEITEM hItem, _In_opt_ void* pData);
  _NODISCARD bool IsExpanded(_In_ HTREEITEM hItem);
  _NODISCARD bool GetRefreshAfterShellCommand() const noexcept { return m_bRefreshAfterShellCommand; };
  void SetRefreshAfterShellCommand(_In_ bool bRefreshAfterShellCommand) noexcept { m_bRefreshAfterShellCommand = bRefreshAfterShellCommand; };
  void SetAutoRefresh(_In_ bool bAutoRefresh);
  _NODISCARD bool GetAutoRefresh() const noexcept { return m_bAutoRefresh; };

protected:
//Methods
  LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam) override;
  BOOL PreTranslateMessage(MSG* pMsg) override;
  void PreSubclassWindow() override;
  virtual bool GetRootItems();
  virtual bool GetChildItems(_In_ HTREEITEM hParentItem);
  virtual SFGAOF GetAttributesOfFlags();
  virtual HRESULT EnumObjects(_In_ HTREEITEM hParentItem, _In_ ATL::CComPtr<IShellFolder>& pParentFolder, _In_ const CPIDL& pidlParent);
  static int CALLBACK CompareProc(_In_ LPARAM lParam1, _In_ LPARAM lParam2, _In_ LPARAM lParamSort);
  virtual CString OnGetItemText(_In_ const CShellItemInfo2* pItem);
  virtual int OnGetItemIcon(_In_ const CShellItemInfo2* pItem, _In_ bool bSelected);
  virtual void OnShowContextMenu(_In_ CPoint point);
  virtual void OnSelectionChanged(_In_ const NMTREEVIEW* pNMTreeView);
  virtual bool CanDisplayFile(_In_z_ LPCTSTR pszFilename);
  virtual bool CanDisplayFolder(_In_z_ LPCTSTR pszFolder);
  virtual bool CanDisplayDrive(_In_ const CString& sDrive);
  virtual bool CanDisplay(_In_ const CPIDL& pidlParent, _In_ const CPIDL& pidl);
  virtual bool HasChildren(_Inout_ CShellItemInfo2& item, _In_ SFGAOF attribs);
  virtual void PrepareTreeItem(_In_ SFGAOF attribs, _Inout_ CString& sItem, _Inout_ CShellItemInfo2& item, _Inout_ TVITEM& tvItem);
  virtual HTREEITEM AddItem(_In_ HTREEITEM hParentItem, _In_ ATL::CComPtr<IShellFolder>& pParentFolder, _In_ const CPIDL& pidl, _In_ SFGAOF attribs, _In_ const CPIDL& pidlParent);
  virtual void OnItemAdded(_In_ HTREEITEM hItem);
  virtual void CreateMonitoringThread(_In_z_ LPCTSTR pszPath);
  static UINT _MonitoringThread(_In_ LPVOID pParam);
  virtual UINT MonitoringThread();
  virtual void KillMonitoringThread();
  virtual bool CanHandleChangeNotifications(_In_z_ LPCTSTR pszPath);

//Member variables
  CPIDL m_RootFolder;
  bool m_bAllowDoShellDefault;
  DWORD m_dwDriveHideFlags;
  DWORD m_dwFileHideFlags;
  DWORD m_dwFolderHideFlags;
  std::vector<CString> m_FilenameMasks;
  COLORREF m_rgbCompressed;
  bool m_bShowCompressedUsingDifferentColor;
  COLORREF m_rgbEncrypted;
  bool m_bShowEncryptedUsingDifferentColor;
  std::vector<HTREEITEM> m_PrevItems;
  std::vector<HTREEITEM> m_NextItems;
  size_t m_nMaxHistory;
  bool m_bUpdatingHistorySelection;
  bool m_bShowSharedUsingDifferentIcon;
  bool m_bShowLinkedItemsUsingDifferentIcon;
  bool m_bKeepChildrenOnCollapse;
  bool m_bShowRootedFolder;
  bool m_bContextMenu;
  bool m_bNoNotify;
  HWND m_hwndRelatedList;
  SHCONTF m_EnumObjectFlags;
  bool m_bRefreshAfterShellCommand;
  ATL::CComPtr<IContextMenu2> m_ContextMenu2;
  CWinThread* m_pMonitoringThread;
  CString m_sMonitoringPath;
  CEvent m_MonitoringTerminateEvent;
  std::atomic<bool> m_bAutoRefresh;
  std::atomic<bool> m_bShowFiles;

//Message handlers
  afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
  afx_msg void OnDestroy();
  afx_msg void OnCustomDraw(NMHDR* pNMHDR, LRESULT* pResult);
  afx_msg void OnDblClk(NMHDR* pNMHDR, LRESULT* pResult);
  afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
  afx_msg void OnItemExpanding(NMHDR* pNMHDR, LRESULT* pResult);
  afx_msg void OnDeleteItem(NMHDR* pNMHDR, LRESULT* pResult);
  afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
  afx_msg void OnSelChanged(NMHDR* pNMHDR, LRESULT* pResult);
  afx_msg LRESULT OnNotificationChange(WPARAM wParam, LPARAM lParam);

  friend class CShellListCtrl2;

  DECLARE_MESSAGE_MAP()
  DECLARE_DYNCREATE(CShellTreeCtrl2)
};


#endif //#ifndef __SHELLTREECTRL2_H__
